/****
 *
 * Exemplo: Como Pensar Recursivamente
 *
 * Seo: 4.6
 *
 * Descrio: Verifica se um valor faz parte de um array
 *
 ****/

#include <stdio.h> /* Funo printf() */

/****
 *
 * EmArray(): Verifica se um nmero inteiro encontra-se
 *            num array de inteiros
 *
 * Parmetros: ar[] (entrada): o array que ser pesquisado
 *             tam (entrada): nmero de elementos do array
 *             num (entrada): o nmero que ser procurado
 *
 * Retorno: ndice do nmero no array, se ele for encontrado
 *          -1, em caso contrrio
 *
 ****/
int EmArray(const int ar[], int tam, int num)
{
   int i;

      /* Compara cada elemento do array com o        */
      /* parmetro 'num'. Se for encontrado um       */
      /* elemento igual a 'num', retorna seu ndice. */
   for (i = 0; i < tam; ++i)
      if (ar[i] == num)
         return i; /* Encontrado um elemento igual */

      /* Valor recebido como parmetro */
      /* no foi encontrado no array   */
   return -1;
}

/****
 *
 * EmArrayRec(): Verifica se um nmero inteiro encontra-se
 *               num array de inteiros (verso recursiva)
 *
 * Parmetros: ar[] (entrada): o array que ser pesquisado
 *             tam (entrada): nmero de elementos do array
 *             inf (entrada): ndice inferior do array
 *             sup (entrada): ndice superior do array
 *
 * Retorno: ndice do nmero no array, se ele for encontrado
 *          -1, em caso contrrio
 *
 ****/
int EmArrayRec(const int ar[], int inf, int sup, int num)
{
      /* Verifica os casos base */
   if (ar[inf] == num)  /* Caso base 1 */
         /* O valor procurado  o primeiro do array */
      return inf; /* Encontrado um elemento igual a 'num' */
   else if (inf >= sup)  /* Caso base 2 */
         /* Quando o ndice inferior do array  maior */
         /* do que ou igual ao seu ndice superior,   */
         /* todo o array j foi examinado             */
      return -1; /* Elemento no foi encontrado */

      /* Caso recursivo: procura no restante do array */
   return EmArrayRec(ar, inf + 1, sup, num);
}

/****
 *
 * EmArrayRec2(): Verifica se um nmero inteiro encontra-se
 *               num array de inteiros (verso recursiva)
 *
 * Parmetros: ar[] (entrada): o array que ser pesquisado
 *             tam (entrada): nmero de elementos do array
 *             num (entrada): o nmero que ser procurado
 *
 * Retorno: 1, se o nmero for encontrado
 *          0, em caso contrrio
 *
 ****/
int EmArrayRec2(const int ar[], int tam, int num)
{
      /* Compara cada elemento do array com o        */
      /* parmetro 'num'. Se for encontrado um       */
      /* elemento igual a 'num', retorna seu ndice. */
   if (!tam) /* Caso base 1 */
      return 0; /* Elemento no foi encontrado */
   else if (ar[0] == num) /* Caso base 2 */
      return 1; /*  */

      /* Caso recursivo: procura no restante do array */
   return EmArrayRec2(ar + 1, tam - 1, num);

      /* Valor recebido como parmetro */
      /* no foi encontrado no array   */
   return -1;
}

/****
 *
 * EmArray2(): Acionadora da funo EmArrayRec()
 *
 * Parmetros: ar[] (entrada): o array que ser pesquisado
 *             tam (entrada): nmero de elementos do array
 *             num (entrada): o nmero que ser procurado
 *
 * Retorno: ndice do nmero no array, se ele for encontrado
 *          -1, em caso contrrio
 *
 ****/
int EmArray2(const int ar[], int tam, int num)
{
   return EmArrayRec(ar, 0, tam - 1, num);
}

/****
 *
 * main(): Testa as funes EmArray() e EmArrayRec()
 *
 * Parmetros: Nenhum
 *
 * Retorno: zero
 *
 ****/
int main(void)
{
   int a[] = {-1, 0, 3, 6, 12, 19},
       nElementos, posicao;

   nElementos = sizeof(a)/sizeof(a[0]);

   printf("\n>>> Usando EmArray() <<< \n");

   posicao = EmArray(a, nElementos, 6);

   if (posicao != -1)
      printf("%d foi encontrado na posicao %d\n", 6, posicao);
   else
      printf("%d nao foi encontrado\n", 6);

   printf("\n>>> Usando EmArrayRec() <<< \n");

   posicao = EmArrayRec(a, 0, nElementos - 1, 6);

   if (posicao != -1)
      printf("%d foi encontrado na posicao %d\n", 6, posicao);
   else
      printf("%d nao foi encontrado\n", 6);

   printf("\n>>> Usando EmArrayRec2() <<< \n");

   if (EmArrayRec2(a, nElementos, 6))
      printf("%d foi encontrado mas nao sei onde\n", 6);
   else
      printf("%d nao foi encontrado\n", 6);

   printf("\n>>> Usando EmArrayRec() <<< \n");

   posicao = EmArrayRec(a, 0, nElementos - 1, 22);

   if (posicao != -1)
      printf("%d foi encontrado na posicao %d\n", 22, posicao);
   else
      printf("%d nao foi encontrado\n", 22);

   printf("\n>>> Usando EmArrayRec() <<< \n");

   posicao = EmArrayRec(a, 0, nElementos - 1, 22);

   if (posicao != -1)
      printf("%d foi encontrado na posicao %d\n", 22, posicao);
   else
      printf("%d nao foi encontrado\n", 22);

   printf("\n>>> Usando EmArrayRec2() <<< \n");

   if (EmArrayRec2(a, nElementos, 22))
      printf("%d foi encontrado mas nao sei onde\n", 22);
   else
      printf("%d nao foi encontrado\n", 22);

   return 0;
}

/****************** Resultado do Programa *****************

>>> Usando EmArray() <<<
6 foi encontrado na posicao 3

>>> Usando EmArrayRec() <<<
6 foi encontrado na posicao 3

>>> Usando EmArrayRec2() <<<
6 foi encontrado mas nao sei onde

>>> Usando EmArrayRec() <<<
22 nao foi encontrado

>>> Usando EmArrayRec() <<<
22 nao foi encontrado

>>> Usando EmArrayRec2() <<<
22 nao foi encontrado

**********************************************************/
