#include <stdio.h>      /* printf() */
#include "Complexos1.h" /* Interface do tipo complexo */

/****
 *
 * IniciaComplexo(): Atribui um valor a um nmero complexo
 *
 * Parmetros:
 *      x, y (entrada) - partes real e imaginria a serem
 *                       atribudas ao nmero complexo
 *
 * Retorno: O resultado da atribuio
 *
 ****/
tComplexo IniciaComplexo(double x, double y)
{
   tComplexo umComplexo;

   umComplexo.pReal = x;
   umComplexo.pImaginaria = y;

   return umComplexo;
}

/****
 *
 * ConjugadoComplexo(): Calcula o conjugado de um nmero complexo
 *
 * Parmetros:
 *      c (entrada) - nmero complexo cujo conjungado ser calculado
 *
 * Retorno: O conjugado do nmero complexo recebido como parmetro
 *
 ****/
tComplexo ConjugadoComplexo(tComplexo c)
{
   tComplexo umComplexo;

   umComplexo.pReal = c.pReal;
   umComplexo.pImaginaria = -c.pImaginaria;

   return umComplexo;
}

/****
 *
 * SomaComplexos(): Soma dois nmeros complexos
 *
 * Parmetros:
 *      c1, c2 (entrada) - os dois nmeros que sero somados
 *
 * Retorno: O resultado da soma
 *
 ****/
tComplexo SomaComplexos(tComplexo c1, tComplexo c2)
{
   tComplexo resultado;

   resultado.pReal = c1.pReal + c2.pReal;
   resultado.pImaginaria = c1.pImaginaria + c2.pImaginaria;

   return resultado;
}

/****
 *
 * SubtraiComplexos(): Subtrai dois nmeros complexos
 *
 * Parmetros:
 *      c1 (entrada) - o minuendo
 *      c2 (entrada) - o subtraendo
 *
 * Retorno: O resultado da subtrao
 *
 ****/
tComplexo SubtraiComplexos(tComplexo c1, tComplexo c2)
{
   tComplexo resultado;

   resultado.pReal = c1.pReal - c2.pReal;
   resultado.pImaginaria = c1.pImaginaria - c2.pImaginaria;

   return resultado;
}

/****
 *
 * MultiplicaComplexos(): Multiplica dois nmeros complexos
 *
 * Parmetros:
 *      c1, c2 (entrada) - os dois nmeros que sero multiplicaados
 *
 * Retorno: O resultado da multiplicao
 *
 ****/
tComplexo MultiplicaComplexos(tComplexo c1, tComplexo c2)
{
   tComplexo resultado;

   resultado.pReal = c1.pReal*c2.pReal - c1.pImaginaria*c2.pImaginaria;
   resultado.pImaginaria = c1.pImaginaria*c2.pReal + c1.pReal*c2.pImaginaria;

   return resultado;
}

/****
 *
 * ExibeComplexo(): Exibe um nmero complexo na tela
 *
 * Parmetros:
 *      c (entrada) - o nmero complexo que ser exibido
 *
 * Retorno: Nada
 *
 ****/
void ExibeComplexo(tComplexo c)
{
   if (c.pImaginaria < 0) {
      printf("%5.2f - %5.2fi", c.pReal, c.pImaginaria);
   } else if (c.pImaginaria > 0) {
      printf("%5.2f + %5.2fi", c.pReal, c.pImaginaria);
   } else {
      printf("%5.2f", c.pReal);
   }
}

