#include <stdio.h>      /* printf() */
#include <stdlib.h>     /* malloc() */
#include "Complexos2.h" /* Interface do tipo tComplexo2 */

    /* Completa a definio do tipo tComplexo2 */
struct complexo {
             double pReal;
             double pImaginaria;
        };

/****
 *
 * CriaComplexo(): Aloca dinamicamente uma varivel do tipo
 *                 apontado por um ponteiro do tipo tComplexo2
 *                 e atribui um valor a ela
 *
 * Parmetros:
 *      x, y (entrada) - partes real e imaginria a serem
 *                       atribudas ao nmero complexo
 *
 * Retorno: O resultado da atribuio
 *
 ****/
tComplexo2 CriaComplexo(double x, double y)
{
   tComplexo2 umComplexo = malloc(sizeof(struct complexo));

   umComplexo->pReal = x;
   umComplexo->pImaginaria = y;

   return umComplexo;
}

/****
 *
 * CriaComplexo(): Libera o espao ocupado uma varivel do tipo
 *                 apontado por um ponteiro do tipo tComplexo2
 *
 * Parmetros: c (entrada e sada) - endereo de um ponteiro para a varivel
 *                                   que ter seu espao liberado
 *
 * Retorno: Nada
 *
 ****/
void DestroiComplexo(tComplexo2 *c)
{
   free(*c);

   *c = NULL;
}

/****
 *
 * ConjugadoComplexo(): Calcula o conjugado de um nmero complexo
 *
 * Parmetros:
 *      c (entrada) - nmero complexo cujo conjungado ser calculado
 *
 * Retorno: O conjugado do nmero complexo recebido como parmetro
 *
 ****/
tComplexo2 ConjugadoComplexo(tComplexo2 c)
{
   tComplexo2 umComplexo = malloc(sizeof(struct complexo));

   umComplexo->pReal = c->pReal;
   umComplexo->pImaginaria = -c->pImaginaria;

   return umComplexo;
}

/****
 *
 * SomaComplexos(): Soma dois nmeros complexos
 *
 * Parmetros:
 *      c1, c2 (entrada) - os dois nmeros que sero somados
 *
 * Retorno: O resultado da soma
 *
 ****/
tComplexo2 SomaComplexos(tComplexo2 c1, tComplexo2 c2)
{
   tComplexo2 resultado = malloc(sizeof(struct complexo));

   resultado->pReal = c1->pReal + c2->pReal;
   resultado->pImaginaria = c1->pImaginaria + c2->pImaginaria;

   return resultado;
}

/****
 *
 * SubtraiComplexos(): Subtrai dois nmeros complexos
 *
 * Parmetros:
 *      c1 (entrada) - o minuendo
 *      c2 (entrada) - o subtraendo
 *
 * Retorno: O resultado da subtrao
 *
 ****/
tComplexo2 SubtraiComplexos(tComplexo2 c1, tComplexo2 c2)
{
   tComplexo2 resultado = malloc(sizeof(struct complexo));

   resultado->pReal = c1->pReal - c2->pReal;
   resultado->pImaginaria = c1->pImaginaria - c2->pImaginaria;

   return resultado;
}

/****
 *
 * MultiplicaComplexos(): Multiplica dois nmeros complexos
 *
 * Parmetros:
 *      c1, c2 (entrada) - os dois nmeros que sero multiplicaados
 *
 * Retorno: O resultado da multiplicao
 *
 ****/
tComplexo2 MultiplicaComplexos(tComplexo2 c1, tComplexo2 c2)
{
   tComplexo2 resultado = malloc(sizeof(struct complexo));

   resultado->pReal = c1->pReal*c2->pReal - c1->pImaginaria*c2->pImaginaria;
   resultado->pImaginaria = c1->pImaginaria*c2->pReal +
                            c1->pReal*c2->pImaginaria;

   return resultado;
}

/****
 *
 * ExibeComplexo(): Exibe um nmero complexo na tela
 *
 * Parmetros:
 *      c (entrada) - o nmero complexo que ser exibido
 *
 * Retorno: Nada
 *
 ****/
void ExibeComplexo(tComplexo2 c)
{
   if (c->pImaginaria < 0) {
      printf("%5.2f - %5.2fi", c->pReal, c->pImaginaria);
   } else if (c->pImaginaria > 0) {
      printf("%5.2f + %5.2fi", c->pReal, c->pImaginaria);
   } else {
      printf("%5.2f", c->pReal);
   }
}
