/****
 *
 * Ttulo: Quando Ser o Fim do Mundo?
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Resolve o problema das Torres de Hani
 *
 * Entrada: Nmero de discos do quebra-cabeas
 *
 * Sada: Instrues informando sobre movimentos dos discos
 *        de uma haste para outra
 *
 ****/

#include <stdio.h>

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf( "\a\n\t>>> %s", "O valor digitado e' "
              "invalido. Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Se for o caso, repreende o usurio por */
      /* ele ter digitado mais do que deveria   */

      /* Verifica se h mensagem a ser apresentada  e  */
      /* apresenta a mensagem adequada para a situao */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n", nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * TorresDeHanoi(): Resolve o problema das Torres de Hani
 *
 * Parmetros:
 *      nDiscos (entrada) - nmero de discos
 *      hasteOrigem (entrada) - haste na qual os discos se
 *                              encontram inicialmente
 *      hasteDestino (entrada) - haste  qual os discos se
 *                               destinam
 *      hasteAuxiliar (entrada) - haste usada como auxiliar
 *
 * Retorno: Nada
 *
 ****/
void TorresDeHanoi( int nDiscos, int hasteOrigem,
                    int hasteDestino, int hasteAuxiliar )

{
   if (nDiscos == 1){ /* Passo 1 do algoritmo */
      printf("Mova o disco 1 da haste %c para a haste %c\n",
             hasteOrigem, hasteDestino); /* Escreve e encerra */
       return;
   }

   /* Move os n-1 discos de A para B usando  */
   /* C como auxiliar (Passo 2 do algoritmo) */
   TorresDeHanoi( nDiscos - 1, hasteOrigem,
                  hasteAuxiliar, hasteDestino );

      /* Move ltimo disco de A para C */
      /* (Passo 3 do algoritmo)        */
   printf( "Mova o disco %d da haste %c para a haste %c\n",
           nDiscos, hasteOrigem, hasteDestino );

      /* Move os n-1 discos de B para C usando A */
      /* como auxiliar (Passo 4 do algoritmo)    */
   TorresDeHanoi( nDiscos - 1, hasteAuxiliar,
                 hasteDestino,hasteOrigem );
}

/****
 *
 * main(): Testa a funo TorresDeHanoi()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int    nDiscos;

      /* Solicita dado ao usurio */
   printf("Introduza o numero de discos: ");

   scanf("%d", &nDiscos); /* No esquea de passar o endereo aqui!*/
   printf("\n\n");

   TorresDeHanoi(nDiscos, 'A', 'C', 'B');

   return 0;
}


/******************* Exemplo de Execuo ******************

Introduza o numero de discos: 4


Mova o disco 1 da haste A para a haste B
Mova o disco 2 da haste A para a haste C
Mova o disco 1 da haste B para a haste C
Mova o disco 3 da haste A para a haste B
Mova o disco 1 da haste C para a haste A
Mova o disco 2 da haste C para a haste B
Mova o disco 1 da haste A para a haste B
Mova o disco 4 da haste A para a haste C
Mova o disco 1 da haste B para a haste C
Mova o disco 2 da haste B para a haste A
Mova o disco 1 da haste C para a haste A
Mova o disco 3 da haste B para a haste C
Mova o disco 1 da haste A para a haste B
Mova o disco 2 da haste A para a haste C
Mova o disco 1 da haste B para a haste C

**********************************************************/

