/****
 *
 * Exemplo: Representao de Polinmios
 *
 * Seo: 7.5.1
 *
 * Descrio: Demonstra representao de polinmios por
 *            meio de listas indexadas
 *
 ****/

#include <stdio.h> /* printf() */

/****
 *
 * SomaPol(): soma dois polinmios representados segundo o
 *            esquema apresentado no texto
 *
 * Parmetros:
 *      A (entrada) - primeiro polinmio
 *      B (entrada) - segundo polinmio
 *      C (sada) - polinmio que representa a soma dos polinmios A e B
 *
 * Retorno: Nada
 *
 ****/
void SomaPol(const int A[], const int B[], int C[])
{
   int m = A[0],  /* Nmero de termos do polinmio A */
       n = B[0],  /* Nmero de termos do polinmio B */
       p = 1,     /* Um ndice do polinmio A */
       q = 1,     /* Um ndice do polinmio B */
       r = 1;     /* Um ndice do polinmio C */

      /* Soma os coeficientes dos termos de mesmo expoente e */
      /* armazena os termos que no possuem o mesmo expoente */
   while ( (p <= 2*m) && (q <= 2*n) ) {
         /* Verifica se os expoentes so iguais */
      if ( A[p] == B[q] ) {
            /* Os expoentes so iguais. Portanto, */
            /* os coeficientes so somados        */
         C[r + 1] = A[p + 1] + B[q + 1];

            /* O termo s ser armazenado em C[] se o  */
            /* resultado da soma for diferente de zero */
         if ( C[r + 1] ) {
            C[r] = A[p]; /* Armazena expoentes em C[] */
            r += 2; /* Avana para o prximo termo de C[] */
         }

         p += 2; /* Avana para o prximo termo de A[] */
         q += 2; /* Avana para o prximo termo de B[] */

         /* Armazena em C[] o termo que tem maior */
         /* expoente e avana-se um termo adiante */
      } else if ( A[p] < B[q] ) {
            /* O termo corrente de B[]  maior do que o termo */
            /* corrente de A[] e ser armazenado em C[]       */
         C[r + 1] = B[q + 1];
         C[r] = B[q];

         q += 2; /* Passa para o prximo termo de B[] */
         r += 2; /* Passa para o prximo termo de C[] */
      } else  { /* A[p] > B[q]. Armazena novo termo em C */
            /* O termo corrente de A[]  maior do que o termo */
            /* corrente de B[] e ser armazenado em C[]       */
         C[r + 1] = A[p + 1];
         C[r] = A[p];

         p += 2; /* Passa para o prximo termo de A[] */
         r += 2; /* Passa para o prximo termo de C[] */
      } /* if */
   } /* while */

   /*                                                   */
   /* Neste ponto, uma das seguintes condies causaram */
   /* o encerramento do lao while acima:               */
   /* (1) Todos os termos de ambos os polinmios foram  */
   /*     levados em considerao. Neste caso, nenhum   */
   /*     dos laos a seguir ser executado.            */
   /* (2) Todos os termos de apenas um dos polinmios   */
   /*     foram levados em considerao. Neste caso,    */
   /*     apenas um dos laos a seguir ser executado.  */
   /*                                                   */

      /* Se houver termos em A[] que no foram levados */
      /* em considerao, armazena-os em C[]           */
   while ( p <= 2*m ) {
      C[r] = A[p];
      C[r+1] = A[p+1];
      p += 2;
      r += 2;
   }

      /* Se houver termos em B[] que no foram levados */
      /* em considerao, armazena-os em C[]           */
   while ( q <= 2*n ) {
      C[r] = B[q];
      C[r+1] = B[q+1];
      q = q+2;
      r = r+2;
   }

      /* Armazena o nmero de termos da soma em C[] */
   C[0] = r / 2;
}

/****
 *
 * EscrevePol(): Escreve na tela um polinmio, representado
 *               segundo o esquema apresentado no texto,
 *               num formato legvel
 *
 * Parmetros: A (entrada) - polinmio a ser escrito
 *
 * Valor de retorno: nenhum
 *
 ****/
void EscrevePol(const int A[])
{
   int i;

   for (i = 1; i <= 2*A[0]; i += 2) {
      if (A[i+1] && A[i+1] != 1)
         printf("%d", A[i+1]);
         if (A[i] && A[i] != 1)
            printf("X^%d", A[i]);
         else if (A[i] == 1)
            printf("X");

      if ( i < 2*(A[0] - 1) )
         printf(" + ");
   }
}

/****
 *
 * main(): Testa as funes de processamento de polinmios
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   int poli[50] = { 2, 1, 2, 0, 3,        /* Polinmio A */
                    3, 2, 1, 1, 5, 0, 3,  /* Polinmio B */
                    2, 10, 3, 4, 9 };     /* Polinmio C */
   int A = 0, B = 5, C = 12, disponivel = 17;

   printf("Primeiro polinomio: ");
   EscrevePol(poli + A);

   printf("\n\nSegundo polinomio: ");
   EscrevePol(poli + B);

   SomaPol(poli + A, poli + B, poli + disponivel);

   printf("\n\nSoma dos polinomios: ");
   EscrevePol(poli + disponivel);

   return 0;
}

/****************** Resultado do Programa *****************

Primeiro polinomio: 2X + 3

Segundo polinomio: X^2 + 5X + 3

Soma dos polinomios: X^2 + 7X + 6

**********************************************************/
