/****
 *
 * Exemplo: Representao de Matrizes Esparsas
 *
 * Seo: 7.5.2
 *
 * Descrio: Demonstra representao de matrizes esparsas
 *            por meio de listas indexadas
 *
 ****/

#include <stdio.h> /* printf() */

/****
 *
 * Transpoe(): Calcula a matriz transposta de uma dada matriz
 *
 * Parmetros:
 *      A (entrada) - matriz a ser transposta escrita na forma esparsa
 *      T (sada) - matriz transposta de A
 *
 * Retorno: Nada
 *
 ****/
void Transpoe(int A[][3], int T[][3])
{
   int m = A[0][0], /* Nmero de linhas de A[][] */
       n = A[0][1], /* Nmero de colunas de A[][] */
       t = A[0][2], /* Nmero de elementos no nulos de A[][] */
       p, /* Uma linha de A[][] */
       q, /* Uma linha de T[][] */
       col; /* Uma coluna de A[][] */

   T[0][0] = n; /* Nmero de linhas de T[][] */
   T[0][1] = m; /* Nmero de colunas de T[][] */
   T[0][2] = t; /* Nmero de elementos no nulos de T[][] */

      /* Se matriz for nula, no h o que fazer aqui */
   if (t <= 0)
      return;

   q = 1; /* Apontador da prxima tripla em T[][] */

      /* Efetua a transposio por colunas */
   for (col = 1; col <= n; col++)
      for (p = 1; p <= t; p++)
         if (A[p][1] == col) {
               /* Coluna correta: armazena tripla em T[][] */
            T[q][0] = A[p][1];
            T[q][1] = A[p][0];
            T[q][2] = A[p][2];
            q++;
         }
}

/****
 *
 * Armazene(): Verifica se o valor calculado de um dado
 *             elemento  diferente de 0 e, se for, armazena
 *             -o na matriz C juntamente com sua linha e sua
 *             coluna
 *
 * Parmetros:
 *      A (sada) - matriz a ser modificada
 *      q (entrada/sada) - ndice onde ser feito o
 *                          armazenamento
 *      linha (entrada) - a respectiva linha
 *      coluna (entrada) - a respectiva coluna
 *      valor (entrada/sada): o valor do elemento
 *
 * Retorno: Nada
 *
 ****/
void Armazene( int A[][3], int *q, int linha, int coluna,
               int *valor )
{
   if (*valor) {
      A[*q][1] = linha;
      A[*q][2] = coluna;
      A[*q][3] = *valor;
      (*q)++;
      *valor = 0;
   }
}

/****
 *
 * Multiplica(): Calcula o produto de duas matrizes
 *               representadas conforme o esquema
 *               apresentado no texto
 *
 * Parmetros:
 *      A, B (entrada) - matrizes usadas no clculo do
 *                       produto
 *      C (sada) - resultado do produto de A por B
 *
 * Retorno: Nada
 *
 ****/
void Multiplica(int A[][3], int B[][3], int C[][3])
{
   int m, n, ta, tb, i, j, p, q, r, soma, coluna,
       inicioLinha, BT[100][3];

   m = A[0][0];
   n = A[0][1];
   ta = A[0][2];
   if (n != B[0][0]) { /* As matrizes so incompatveis. */
      printf("Matrizes incompativeis");
      return;  /* No h nada mais a fazer */
   } else {
      p = B[0][1];
      tb = B[0][2];
   } /* if */

   Transpoe(B, BT);
   i = 1;
   q = 1;
   inicioLinha = 1;
   r = A[i][0];
   BT[ta+1][0] = n + 1; /* ??? */
   BT[tb+1][0] = p + 1;
   soma = 0;
   BT[tb+1][1] = 0;

   while (i <= ta) { /* gera linha r de C */
      coluna = BT[1][0];
      j = 1;

         /* Multiplica linha de A por coluna de B */
      while (j <= tb + 1) {
         if (A[i][0] != r) { /* final da linha r de A */
            Armazene(C, &q, r, coluna, &soma);
            i = inicioLinha;

               /* Passa para prxima coluna de B */
            while (BT[j][0] == coluna) j++;
               coluna = BT[j][0];
         } else if ( BT[j][0] != coluna) {
            Armazene(C, &q, r, coluna, &soma);

               /* Prepara para multiplicar a */
               /* linha r com prxima coluna */
            i = inicioLinha;
            coluna = BT[j][0];
         } else if (A[i][1] < BT[j][1])
            i++; /* Avana para o prximo termo na linha r */
         else if (A[i][1] == BT[j][1]) {
            soma += A[i][2]*BT[j][2];
            i++;
            j++;
         } else
            j++; /* Avana para prximo termo na coluna */
      } /* while */

      while (A[i][0] == r) /* Avana para prxima linha */
         i++;

      inicioLinha = i;
      r = A[i][0];
   }  /* while */

   C[0][0] = m;
   C[0][1] = p;
   C[0][2] = q - 1;
}

/****
 *
 * ValorElemento(): Determina o valor de um elemento de uma
 *                  matriz esparsa, representada segundo o
 *                  esquema apresentado no texto
 *
 * Parmetros:
 *      A (entrada): a matriz
 *      linha (entrada): a respectiva linha
 *      coluna (entrada): a respectiva coluna
 *
 * Retorno: o valor do elemento (linha, coluna) da matriz
 *
 ****/
int ValorElemento(int A[][3], int linha, int coluna)
{
   int n,i;

   n = A[0][2]; /* numero de elementos no nulos de A */

   for (i = 1; i <= n; i++)
      if ( (A[i][0] == linha) && (A[i][1] == coluna) )
         return A[i][2];

   return 0;
}

/****
 *
 * EscreveMatriz(): Escreve na tela uma matriz esparsa,
 *                  representada segundo o esquema apresentado
 *                  no texto, num formato legvel
 *
 * Parmetro: A (entrada): matriz a ser escrita
 *
 * Valor de retorno: nenhum
 *
 ****/
void EscreveMatriz(int A[][3])
{
   int m,n,linha,coluna;

   m = A[0][0]; /* nmero de linhas de A */
   n = A[0][1]; /* nmero de colunas de A */

   for (linha = 1; linha <= m; linha++)
      for (coluna = 1; coluna <= n; coluna++) {
         if (coluna == 1)
            printf("\n");
         printf("%d\t", ValorElemento(A, linha, coluna));
      }
}

/****
 *
 * main(): Testa as funes de processamento de matrizes esparsas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   int A[7][3] = {
                   {5, 5, 6},
                   {1, 1, 10},
                   {1, 2, 9},
                   {2, 3, 3},
                   {2, 4, 8},
                   {3, 1, 6},
                   {4, 4, 4}
                 };
   int B[7][3] = { {5, 5, 6},
                   {1, 1, 10},
                   {1, 2, 9},
                   {2, 3, 3},
                   {2, 4, 8},
                   {3, 1, 6},
                   {4, 4, 4} };

   int C[7][3], D[26][3];

   printf("Primeira matriz: \n");
   EscreveMatriz(A);

   printf("\n\nSegunda matriz: \n");
   EscreveMatriz(B);

   Transpoe(A, C);
   printf("\n\nTransposta da primeira matriz: \n");
   EscreveMatriz(C);

   Multiplica(A, B, D);
   printf("\n\nProduto das matrizes: \n");
   EscreveMatriz(D);

   return 0;
}

/****************** Resultado do Programa *****************

Primeira matriz:

10      9       0       0       0
0       0       3       8       0
6       0       0       0       0
0       0       0       4       0
0       0       0       0       0

Segunda matriz:

10      9       0       0       0
0       0       3       8       0
6       0       0       0       0
0       0       0       4       0
0       0       0       0       0

Transposta da primeira matriz:

10      0       6       0       0
9       0       0       0       0
0       3       0       0       0
0       8       0       4       0
0       0       0       0       0

Produto das matrizes:

0       0       0       0       0
0       0       0       0       0
0       0       0       0       0
1       0       0       0       0
0       0       0       0       0

**********************************************************/
