/****
 *
 * Ttulo: Fazendo, Desfazendo e Refazendo
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Gerencia uma lista de compras permitindo
 *            desfazer e refazer operaes
 *
 * Entrada: Itens (strings) e opes escolhidas pelo usurio
 *
 * Sada: A lista criada pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada */
#include <string.h>   /* Processamento de strings */
#include "Lista.h"    /* Interface do tipo lista  */
#include "Leitura.h"  /* Funes de leitura       */
#include "ASSEGURA.h" /* Macro ASSEGURA           */

/***************** Constantes Simblicas ******************/

#define MAX_OPCOES 20 /* Nmero mximo de caracteres que */
                      /* representam opes de menu      */

/***************** Definies de Funes ******************/

/****
 *
 * ApresentaMenu(): Apresenta o menu de opes do programa
 *                  de acordo com a lista recebida como parmetro
 *
 * Parmetros:
 *     *lista (entrada) - uma lista
 *
 * Retorno: Endereo de um string cujos caracteres
 *          representam as opes que o usurio poder escolher
 *
 ****/
char *ApresentaMenu( const tListaCompras *lista )
{
   static char opcoes[MAX_OPCOES + 1];

      /* O array opcoes[] armazenar um string cujos caracteres representam  */
      /* as opes que o usurio poder escolher. Aqui, ele recebe as opes */
      /* de encerramento que so sempre disponveis. Isso no pode ser feito */
      /* como iniciao pois o array tem durao fixa                        */
   strcpy(opcoes, "Ee");

   printf( "\n\n\t*** Opcoes ***" ); /* Apresenta o cabealho do menu */

      /****************************************************/
      /* Apresenta o menu de acordo com o estado da lista */
      /****************************************************/

   if (!EstaCheia(lista)) {
         /* Se a lista no est cheia, o usurio pode acrescentar mais itens */
      printf( "\n\t[A]crescenta" );
      strcat(opcoes, "Aa");

      if (Comprimento(lista) > 1) {
            /* Se a lista tiver mais de um elemento, */
            /* o usurio pode inserir um elemento    */
         printf( "\n\t[I]nsere" );
         strcat(opcoes, "Ii");
      }
   }

      /* Se a lista no estiver vazia, o usurio */
      /* pode remover e modificar um elemento    */
   if (!EstaVazia(lista)) {
      printf( "\n\t[R]emove" );
      printf( "\n\t[M]odifica" );

      strcat(opcoes, "RrMm");
   }

      /* Apresenta a ltima opo e o prompt */
   printf( "\n\t[E]ncerra o programa\n\nEscolha sua opcao >>> " );

   return opcoes;
}

/****
 *
 * ApresentaLista(): Exibe uma lista na tela
 *
 * Parmetros:
 *      *aLista (entrada) - a lista que ser exibida na tela
 *
 * Retorno: Nada
 *
 ****/
void ApresentaLista(const tListaCompras *aLista)
{
   int tamanho = Comprimento(aLista);

   if (!tamanho) {
      printf("\n\t*** Lista vazia ***\n");
      return;
   }

   printf("\n\t*** Lista ***\n");

   for (int i = 0; i < tamanho; ++i)
      printf( "\n\t   %2d. %s", i + 1, ObtemItem(aLista, i) );
}

/****
 *
 * main(): Gerencia uma lista de compras
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int           opcao, /* Uma opo escolhida pelo usurio */
                 pos, /* Uma posio na lista */
                 nItens; /* Nmero de itens da lista */
   tListaCompras lista; /* A lista de compras */
   tItemCompra   umItem; /* Um item da lista de compras */
   char         *removido; /* Um item removido da lista */
   char         *opcoes; /* Aponta para o string contendo as   */
                         /* opes que o usurio pode escolher */

   IniciaLista(&lista); /* Inicia a lista de compras  */

      /* Apresenta o programa ao usurio e explica seu funcionamento */
   printf( "\n\t>>> Este programa gerencia uma lista de compras\n" );

   while (1) {
      ApresentaLista(&lista); /* Apresenta a lista */

         /* Alerta o usurio quando a lista est cheia */
      if (EstaCheia(&lista))
         printf("\n*** A lista esta' cheia ***\n");

         /* Apresenta o menu e obtm as opes disponveis */
      opcoes = ApresentaMenu(&lista);

         /* L a opo do usurio (o prompt j foi apresentado ) */
      opcao = LeOpcao(opcoes);

         /*  melhor tratar a opo de encerramento aqui */
      if (opcao == 'E' || opcao == 'e')
         break; /* Usurio quer encerrar o programa */

         /* Processa as demais opes */
      switch (opcao) {
         case 'A': /* Acrescenta */
         case 'a':
            ASSEGURA(!EstaCheia(&lista), "Lista cheia");

            printf("\nDigite o nome do item >>> ");
            LeString( (char *) umItem, MAX_ITEM );
            AcrescentaItem(&lista, umItem);
            break;
         case 'I': /* Insere */
         case 'i':
            ASSEGURA(!EstaCheia(&lista), "Lista cheia");

            nItens = Comprimento(&lista);

            if (nItens > 1) {
               printf( "\nInforme a posicao entre 1 e %d >>> ", nItens );
               pos = LeNaturalPositivo();
            } else {
               pos = 1;
            }

            if (pos > nItens) {
               printf( "\n>>> A posicao deveria ser menor "
                       "do que %d. Nao havera' insercao.", nItens + 1 );
            } else {
                  /* A correo a seguir  necessria porque usurio normal */
                  /* conta a partir de 1, mas programador de C conta a      */
                  /* partir de zero                                         */
               --pos;

               printf("\nDigite o nome do item >>> ");
               LeString( (char *) umItem, MAX_ITEM );

               InsereItem(&lista, umItem, pos);
            }
            break;
         case 'R': /* Remove */
         case 'r':
            ASSEGURA(!EstaVazia(&lista), "Lista vazia");

            nItens = Comprimento(&lista);
            printf( "\nInforme a posicao entre 1 e %d >>> ", nItens );
            pos = LeNaturalPositivo();

            if (pos > nItens) {
               printf( "\n>>> A posicao deveria ser menor "
                       "do que %d. Nao havera' remocao.", nItens + 1 );
            } else {
                  /* A correo a seguir  necessria porque usurio normal */
                  /* conta a partir de 1, mas programador de C conta a      */
                  /* partir de zero                                         */
               --pos;

               removido = RemoveItem(&lista, pos);

               printf( "\n>>> \"%s\" foi removido da lista\n", removido );
            }
            break;
         case 'M': /* Modifica */
         case 'm':
            ASSEGURA(!EstaVazia(&lista), "Lista vazia");

            nItens = Comprimento(&lista);
            printf( "\nInforme a posicao entre 1 e %d >>> ", nItens );
            pos = LeNaturalPositivo();

            if (pos > nItens) {
               printf( "\n>>> A posicao deveria ser menor "
                       "do que %d. Nao havera' modificacao.", nItens + 1 );
            } else {
                  /* A correo a seguir  necessria porque usurio normal */
                  /* conta a partir de 1, mas programador de C conta a      */
                  /* partir de zero                                         */
               --pos;

               printf( "\nDigite o novo nome do item \""
                       "%s\" >>> ", ObtemItem(&lista, pos) );
               LeString( (char *) umItem, MAX_ITEM );

               AlteraItem(&lista, umItem, pos);
            }
            break;
         default:
            ASSEGURA(1, "Ocorreu um erro impossivel");
            break;
      }
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa. Boas compras!\n" );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa gerencia uma lista de compras

        *** Lista vazia ***

        *** Opcoes ***
        [A]crescenta
        [E]ncerra o programa

Escolha sua opcao >>> a

Digite o nome do item >>> Arroz

        *** Lista ***

            1. Arroz

        *** Opcoes ***
        [A]crescenta
        [R]emove
        [M]odifica
        [E]ncerra o programa

Escolha sua opcao >>> a

Digite o nome do item >>> Feijao

        *** Lista ***

            1. Arroz
            2. Feijao

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [E]ncerra o programa

Escolha sua opcao >>> i

Informe a posicao entre 1 e 2 >>> 2

Digite o nome do item >>> Acucar

        *** Lista ***

            1. Arroz
            2. Acucar
            3. Feijao

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [E]ncerra o programa

Escolha sua opcao >>> m

Informe a posicao entre 1 e 2 >>> 2

Digite o novo nome do item "Acucar" >>> Adocante

        *** Lista ***

            1. Arroz
            2. Adocante
            3. Feijao

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [E]ncerra o programa

Escolha sua opcao >>> r

Informe a posicao entre 1 e 3 >>> 1

>>> "Arroz" foi removido da lista

        *** Lista ***

            1. Adocante
            2. Feijao

        *** Opcoes ***
        [A]crescenta
        [I]nsere
        [R]emove
        [M]odifica
        [E]ncerra o programa

Escolha sua opcao >>> e

        >>> Obrigado por usar este programa. Boas compras!

**********************************************************/
