#include <stdio.h>      /* Funes de Entrada/Sada  */
#include <stdlib.h>     /* Funo exit()             */
#include "Pilha.h"      /* Declaraes do tipo pilha */
#include "ASSEGURA.h"   /* Macro ASSEGURA            */

/**************** Definies de funes ******************/

/****
 *
 * PilhaIdxCheia(): Verifica se uma pilha est cheia; i.e., se no
 *               h mais espao para um novo elemento no array
 *
 * Parmetros:
 *      *p (entrada) - ponteiro para a pilha
 *
 * Retorno: 1, se a pilha estiver cheia; 0, em caso contrrio
 *
 * Observaes: Esta funo faz parte da implementao de
 *              pilha e no da abstrao pilha. Por isso,
 *              ela deve ser definida como static.
 *
 ****/
static int PilhaIdxCheia(const tPilhaIdx *p)
{
   return p->topo >= MAX_ELEMENTOS - 1;
}

/****
 *
 * CriaPilhaIdx(): Inicializa uma pilha
 *
 * Parmetros:
 *      *p (sada) - a pilha que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void CriaPilhaIdx(tPilhaIdx *p)
{
   p->topo = -1;
}

/****
 *
 * PilhaIdxVazia(): Verfica se uma pilha est vazia
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: 1 se a pilha recebida como parmetro estiver vazia;
 *          0, em caso contrrio
 *
 ****/
int PilhaIdxVazia(const tPilhaIdx *p)
{
   return p->topo == -1;
}

/****
 *
 * ElementoTopoIdx(): Obtm o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada) - ponteiro para a pilha
 *
 * Retorno: O elemento do topo da pilha recebida como parmetro
 *
 * Observao: Se a pilha estiver vazia, causa aborto de programa
 *
 ****/
tItemPilha ElementoTopoIdx(const tPilhaIdx *p)
{
   ASSEGURA(!PilhaIdxVazia(p), "Erro: Pilha vazia.");

   return p->itens[p->topo];
}

/****
 *
 * EmpilhaIdx(): EmpilhaIdx um item numa pilha
 *
 * Parmetros:
 *      p (entrada e sada) - ponteiro para a pilha na qual
 *                             ocorrer o empilhamento
 *      item (entrada) - ponteiro para o contedo
 *                       que ser empilhado
 *
 * Retorno: Nada
 *
 * Observao: Se a pilha estiver cheia, causa aborto
 *
 ****/
void EmpilhaIdx(tPilhaIdx *p, const tItemPilha *item)
{
   ASSEGURA(!PilhaIdxCheia(p), "Erro: Pilha cheia.");

   p->itens[++p->topo] = *item;
}

/****
 *
 * DesempilhaIdx(): DesempilhaIdx o elemento do topo de uma pilha
 *
 * Parmetros:
 *      p (entrada e sada ) - ponteiro para a pilha na qual
 *                              ocorrer o empilhamento
 *
 * Retorno:  elemento do topo da pilha apontada por p
 *
 * Observao: Se a pilha estiver vazia, causa aborto
 *
 ****/
tItemPilha DesempilhaIdx(tPilhaIdx *p)
{
   ASSEGURA(!PilhaIdxVazia(p), "Erro: Pilha vazia");

   return p->itens[p->topo--];
}
