/****
 *
 * Arquivo: Fila1.c
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Implementao de fila usando array esttico linear
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>     /* Funes de Entrada/Sada  */
#include <stdlib.h>    /* Funo exit()             */
#include "Fila.h"      /* Declaraes do tipo fila  */
#include "ASSEGURA.h"  /* Macro ASSEGURA            */

/***************** Definies de Funes ******************/

/****
 *
 * FilaIdxCheia(): Verifica se uma fila est repleta e, se for
 *              o caso, promove o deslocamento de elementos
 *              de modo a criar espao para insero
 *
 * Parmetros:
 *      f (entrada e sada) - a fila
 *
 * Retorno: 1, se a fila estiver repleta
 *          0, em caso contrrio
 *
 * Observao: Esta funo faz parte da implementao de fila e
 *             no da abstrao fila. Por isso, ela deve ser
 *             definida como static
 *
 ****/
static int FilaIdxCheia(tFilaIdx *f)
{
   int i, deslocamento;

   if (f->fundo < MAX_ELEMENTOS - 1)
      return 0; /* H espao no final do array */

   if (f->frente == -1 && f->fundo == MAX_ELEMENTOS - 1)
      return 1; /* No h mais espao no array */

      /* Neste ponto, sabe-se que h espao, mas   */
      /* necessrio mover os elementos da fila para */
      /* o incio do array. O deslocamento a ser    */
      /* feito em cada elemento  dado por:         */
   deslocamento = MAX_ELEMENTOS - (f->fundo - f->frente);

      /* O lao for a seguir desloca cada elemento */
      /* da fila para o incio do array            */
   for (i = f->frente + 1; i <= f->fundo; ++i)
      f->itens[i - deslocamento] = f->itens[i];

      /* A frente e o fundo da fila tambm */
      /* precisam ser deslocados           */
   f->frente -= deslocamento;
   f->fundo -= deslocamento;

      /* A fila foi deslocada e agora     */
      /* podem-se acrescentar novos itens */
   return 0;
}

/****
 *
 * CriaFilaIdx(): Inicia a fila recebida como parmetro
 *
 * Parmetros:
 *      f (sada) - a fila a ser inniciada
 *
 * Retorno: Nada
 *
 ****/
void CriaFilaIdx(tFilaIdx *f)
{
   f->frente = f->fundo = -1;
}

/****
 *
 * FilaIdxVazia(): Verifica se a fila recebida como parmetro est vazia
 *
 * Parmetros:
 *      f (entrada) - a fila
 *
 * Retorno: 1, se a fila estiver vazia; 0, em caso contrrio
 *
 ****/
int  FilaIdxVazia(const tFilaIdx *f)
{
   return f->frente == f->fundo;
}

/****
 *
 * ElementoFrenteIdx(): Retorna o elemento que se encontra na frente de uma fila
 *
 * Parmetros:
 *      f (entrada) - a fila
 *
 * Retorno: O elemento que se encontra na frente da fila
 *
 * Observao: Se a fila estiver vazia, causa o aborto do programa
 *
 ****/
tItemFila ElementoFrenteIdx(const tFilaIdx *f)
{
   ASSEGURA(!FilaIdxVazia(f), "Erro: Fila vazia");

   return f->itens[f->frente + 1];
}

/****
 *
 * EnfileiraIdx(): Enfileira um item numa fila
 *
 * Parmetros:
 *      f (entrada e sada) - a fila na qual ser feita o acrscimo
 *      item (entrada) - o item que ser acrescentado
 *
 * Retorno: Nada.
 *
 * Observao: Se a fila estiver cheia, causa o aborto do programa
 *
 ****/
void EnfileiraIdx(tFilaIdx *f, const tItemFila *item)
{
   ASSEGURA(!FilaIdxCheia(f), "Erro: Fila cheia.");

   ++f->fundo;

   f->itens[f->fundo] = *item;
}

/****
 *
 * DesenfileiraIdx(): Retira o elemento que se encontra na
 *           frente de uma fila
 *
 * Parmetros:
 *      f (entrada e sada) - a fila na qual ser feita a retirada
 *
 * Retorno: O elemento que se encontra na frente da fila
 *
 * Observao: Se a fila estiver vazia, causa o aborto do programa
 *
 ****/
tItemFila DesenfileiraIdx(tFilaIdx *f)
{
   ASSEGURA(!FilaIdxVazia(f), "Erro: Fila vazia.");

   ++f->frente;

   return f->itens[f->frente];
}
