/****
 *
 * Ttulo: Primos
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada          */
#include <string.h>   /* Processamento de strings */
#include "Lista.h"    /* Interface do mdulo      */
#include "ASSEGURA.h" /* Macro ASSEGURA           */

/***************** Definies de Funes ******************/

/****
 *
 * EstaCheia(): Verifica se uma lista ordenada est cheia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver cheia; 0, em caso contrario
 *
 ****/
int EstaCheia(const tLista *lista)
{
   return lista->nElementos >= MAX_ELEMENTOS;
}

/****
 *
 * EstaVazia(): Verifica se uma lista ordenada est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVazia(const tLista *lista)
{
   return lista->nElementos == 0;
}

/****
 *
 * ApresentaLista(): Exibe uma lista na tela
 *
 * Parmetros:
 *      *aLista (entrada) - a lista que ser exibida na tela
 *
 * Retorno: Nada
 *
 ****/
void ApresentaLista(const tLista *aLista)
{
   if (EstaVazia(aLista)) {
      printf("\n\t*** Lista vazia ***\n");
      return;
   }

   printf("\n\t*** Lista ***\n");

   for (int i = 0; i < aLista->nElementos; ++i) {
      printf( "\n\t   %2d. %s", i + 1,
             (char *) aLista->itens[i] );
   }
}

/****
 *
 * IniciaLista(): Inicia uma lista ordenada
 *
 * Parmetros: lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 ****/
void IniciaLista(tLista *lista)
{
   lista->nElementos = 0;
}

/****
 *
 * Comprimento(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento ser
 *                                calculado
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
int Comprimento(const tLista *lista)
{
   return lista->nElementos; /* No h o que calcular */
}

/****
 *
 * ConsultaElemento(): Consulta o elemento numa dada posio numa
 *                     lista ordenada
 *
 * Parmetros:
 *     *lista (entrada) - lista que contm o elemento desejado
 *      pos (entrada) - a posio do elemento
 *
 * Retorno: O elemento desejado
 *
 ****/
char *ConsultaElemento(const tLista *lista, int pos)
{
   ASSEGURA( pos >= 0 && pos < lista->nElementos,
             "Elemento inexistente" );

   return (char *) lista->itens[pos];
}

/****
 *
 * AlteraValor(): Altera o contedo do elemento numa dada posio
 *                  de uma lista ordenada
 *
 * Parmetros:
 *      lista (sada) - lista que contm o elemento desejado
 *      valor (entrada) - o valor que ser armazenado
 *      pos (entrada) - a posio do elemento
 *
 * Retorno: Nada
 *
 ****/
void AlteraValor(tLista *lista, tItem valor, int pos)
{
   ASSEGURA( pos >= 0 && pos < lista->nElementos,
             "Elemento inexistente" );

   strcpy((char *)lista->itens[pos], (char *)valor);
}

/****
 *
 * InsereElemento(): Insere um novo elemento numa dada posio
 *                   de uma lista ordenada
 *
 * Parmetros:
 *      lista (entrada/sada) - lista na qual ser feita a
 *                              insero
 *      elemento (entrada) - valor do elemento que ser
 *                           inserido
 *      pos (entrada) - a posio de insero
 *
 * Retorno: Nada
 *
 ****/
void InsereElemento(tLista *lista, tItem elemento, int pos)
{
   ASSEGURA(!EstaCheia(lista), "A lista esta cheia");

      /* Abre espao para o novo elemento */
   for (int i = lista->nElementos; i >= pos; --i) {
      strcpy( (char *)lista->itens[i + 1],
              (char *)lista->itens[i] );
   }

      /* Insere o novo elemento */
   strcpy((char *)lista->itens[pos], (char *)elemento);

   ++lista->nElementos; /* O tamanho da lista aumentou */
}

/****
 *
 * AcrescentaElemento(): Acrescenta um novo elemento ao final
 *                       de uma lista ordenada
 *
 * Parmetros:
 *      *lista (sada) - lista onde ser feita o acrscimo
 *       elemento (entrada) - valor do elemento que ser
 *                            acrescentado
 *
 * Retorno: Nada
 *
 ****/
void AcrescentaElemento(tLista *lista, tItem elemento)
{
   ASSEGURA(!EstaCheia(lista), "A lista esta cheia");

      /* Acrescenta um novo elemento ao final da lista */
   strcpy( (char *)lista->itens[lista->nElementos],
           (char *)elemento );

   ++lista->nElementos; /* O tamanho da lista aumentou */
}

/****
 *
 * RemoveElemento(): Remove um elemento numa dada posio
 *                   de uma lista ordenada
 *
 * Parmetros:
 *      *lista (entrada/sada) - lista onde ser feita a
 *                               remoo
 *       pos (entrada) - posio do elemento que ser removido
 *
 * Retorno: Endereo do item removido
 *
 ****/
char *RemoveElemento(tLista *lista, int pos)
{
   int i;
   static tItem itemRemovido;

   ASSEGURA(!EstaCheia(lista), "A lista esta cheia");

   strcpy( (char *)itemRemovido, (char *)lista->itens[pos] );

      /* Remover um elemento significa mover cada elemento  */
      /* uma posio para trs a partir do prximo elemento */
      /* adiante daquele que ser  removido                 */
   for (i = pos; i < lista->nElementos; i++)
      strcpy( (char *)lista->itens[i],
              (char *)lista->itens[i + 1] );

   --lista->nElementos; /* O tamanho da lista diminuiu */

   return (char *)itemRemovido;
}


