/****
 *
 * Ttulo: LeituraFacil
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 15/07/2012
 * ltima modificao: 10/10/2013
 *
 * Descrio: Implementao da biblioteca LeituraFacil
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* Entrada e sada             */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */
#include "leitura.h" /* Interface da biblioteca */
                     /* LeituraFacil            */

/******************** Funes de Locais *******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/******************** Funes de Globais ******************/

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      printf( "\a\n\t>>>Erro de leitura. "
              "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeNatural(): L um nmero natural via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1 || num < 0) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeNaturalPositivo(): L um nmero natural positivo via
 *                      teclado e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNaturalPositivo(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1 || num < 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * LeReal(): L um valor do tipo double via teclado
 *           deixando o buffer associado ao teclado
 *           sem caracteres remanescentes
 *
 * Parmetros: Nenhum
 *
 * Retorno: O valor lido e validado
 *
 ****/
double LeReal(void)
{
   int    teste, /* Armazena o retorno de scanf() */
          nResto = 0; /* Nmero de caracteres */
                      /* excedentes           */
   double valorLido; /* O valor digitado pelo usurio */

      /* O lao encerra quando o usurio se comporta bem */
   while (1) {
         /* Tenta ler um valor do tipo double */
      teste = scanf("%lf", &valorLido);

         /* Se no ocorreu erro de leitura ou de final */
         /* de arquivo, h caracteres remanescentes    */
         /* que precisam ser removidos                 */
      if (teste != EOF) {
         nResto = LimpaBuffer();
      }

         /* Se o valor retornado por scanf() */
         /* foi 1, a leitura foi OK          */
      if (teste == 1) {
         break; /* Leitura foi nota 10 */
      } else { /* Usurio foi mal comportado */
         printf( "\a\n\t>>> O valor digitado e' invalido. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* Retorna um valor vlido do tipo double */
   return valorLido;
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         printf( "\a\n\t>>> Opcao incorreta. "
                "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

