/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <string.h>   /* Funes strXXX()        */
#include <stdlib.h>   /* malloc()                */
#include "ASSEGURA.h" /* Macro ASSEGURA()        */
#include "Lista.h"    /* Interface do tipo lista */

/****************** Complemento de Tipo *******************/

struct rotListaIdxD {
   tElemento *elementos; /* Ponteiro para o array que */
                         /* contm os elementos       */
   int        nElementos; /* Nmero de elementos */
};

/***************** Definies de Funes ******************/

/****
 *
 * CriaListaIdxD(): Cria uma lista
 *
 * Parmetros: No h
 *
 * Retorno: Endereo da lista criada
 *
 ****/
tListaIdxD CriaListaIdxD(void)
{
   tListaIdxD umaLista;

      /* Aloca espao para a estrutura que armazena a lista */
   umaLista = malloc(sizeof(struct rotListaIdxD));

      /* Garante que a alocao realmente ocorreu */
   ASSEGURA(umaLista, "Nao foi possivel alocar lista");

      /* Inicialmente a lista est vazia */
   umaLista->nElementos = 0;
   umaLista->elementos = NULL;

   return umaLista;
}

/****
 *
 * DestroiListaIdxD(): Libera o espao ocupado pela lista
 *
 * Parmetros: lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaIdxD(tListaIdxD lista)
{
      /* Se a lista no estiver vazia, libera o espao */
      /* ocupado pelo array que contm os elementos    */
   if (lista->nElementos) {
      free(lista->elementos);
   }

      /* Libera a estrutura que representa a lista */
   free(lista);
}

/****
 *
 * ComprimentoListaIdxD(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento
 *                                ser calculado
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
int ComprimentoListaIdxD(tListaIdxD lista)
{
   return lista->nElementos; /* No h o que calcular */
}

/****
 *
 * ObtemElementoListaIdxD(): Obtm o elemento de uma lista que se
 *                  encontra numa dada posio
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *      indice (entrada) - o elemento
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
tElemento ObtemElementoListaIdxD(tListaIdxD lista, int indice)
{
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Elemento inexistente" );

   return lista->elementos[indice];
}

/****
 *
 * InsereListaIdxD(): Insere um novo elemento numa lista numa
 *                   posio especificada
 *
 * Parmetros:
 *      lista (entrada e sada) - lista na qual ser feita
 *                                a insero
 *      item (entrada) - elemento que ser inserido
 *      indice (entrada) - posio de insero
 *
 * Retorno: Nada
 *
 ****/
void InsereListaIdxD(tListaIdxD lista, tElemento item, int indice)
{
   tElemento *novoArray; /* Ponteiro para o array */
                         /* redimensionado        */
   int        nBytesArray; /* Nmero de bytes do  */
                           /* array aps insero */

      /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice <= lista->nElementos,
             "\nPosicao de insercao inexistente\n");

      /*                                                */
      /* Nessa abordagem, o tamanho da lista coincide   */
      /* com o tamanho do array, de modo que nunca h   */
      /* espao desocupado no array e de modo que       */
      /* redimensionamento sempre se faz necessrio     */
      /* quando ocorre insero ou remoo de elementos */
      /*                                                */

      /* Calcula o novo tamanho do array em bytes */
   nBytesArray = (lista->nElementos + 1)*sizeof(tElemento);

      /* Tenta redimensionar o array */
   novoArray = realloc(lista->elementos, nBytesArray);

      /* Garante que houve realocao de memria */
   ASSEGURA(novoArray, "Nao houve redimensionamento");

      /* O ponteiro que apontava para o incio */
      /* do array pode no ser mais vlido.    */
      /* Portanto,  preciso atualiz-lo.      */
   lista->elementos = novoArray;

      /* Abre espao para o novo elemento */
   for (int i = lista->nElementos - 1; i >= indice; --i)
      lista->elementos[i + 1] = lista->elementos[i];

      /* Insere o novo elemento */
   lista->elementos[indice] = item;

   lista->nElementos++; /* O tamanho da lista aumentou */
}

/****
 *
 * RemoveListaIdxD(): Remove um elemento numa dada posio
 *                   de uma lista sem ordenao
 *
 * Parmetros:
 *      lista (entrada/sada) - lista onde ser feita a
 *                              remoo
 *      indice (entrada) - posio do elemento que ser removido
 *
 * Retorno: O item removido
 *
 ****/
tElemento RemoveListaIdxD(tListaIdxD lista, int indice)
{
   tElemento  itemRemovido; /* Armazena o item removido */
   tElemento *novoArray; /* Ponteiro para o array */
                         /* redimensionado        */
   int        nBytesArray; /* Nmero de bytes do  */
                           /* array aps insero */

         /* Garante que o ndice  vlido */
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Posicao de remocao inexistente" );

      /* Guarda o valor do elemento a ser removido */
   itemRemovido = lista->elementos[indice];

      /* Remover um elemento significa mover cada   */
      /* elemento uma posio para trs a partir    */
      /* do sucessor do elemento que ser  removido */
   for (int i = indice; i < lista->nElementos - 1; i++)
      lista->elementos[i] = lista->elementos[i + 1];

   --lista->nElementos; /* O tamanho da lista diminuiu */

      /*                                                */
      /* Nessa abordagem, o tamanho da lista coincide   */
      /* com o tamanho do array, de modo que nunca h   */
      /* espao desocupado no array e de modo que       */
      /* redimensionamento sempre se faz necessrio     */
      /* quando ocorre insero ou remoo de elementos */
      /*                                                */

      /* Calcula o novo tamanho do array em bytes */
   nBytesArray = lista->nElementos*sizeof(tElemento);

      /* Tenta redimensionar o array */
   novoArray = realloc(lista->elementos, nBytesArray);

      /* Garante que houve realocao de memria */
   ASSEGURA(novoArray, "Nao houve redimensionamento");

      /* O ponteiro que apontava para o incio */
      /* do array pode no ser mais vlido.    */
      /* Portanto,  preciso atualiz-lo.      */
   lista->elementos = novoArray;

   return itemRemovido;
}

/****
 *
 * EstaVazia(): Verifica se uma lista sem ordenao est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVazia(tListaIdxD lista)
{
   return lista->nElementos == 0;
}

/****
 *
 * BuscaSequencialListaIdxD(): Efetua uma busca sequencial numa
 *                    lista de elementos do tipo tAluno
 *                    implementada usando array dinmico
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       matrProcurada (entrada) - valor da matrcula do elemento
 *                                  que ser procurado
 *
 * Retorno: ndice do primeiro elemento no array que casa com
 *          o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 ****/
int BuscaSequencialListaIdxD(tListaIdxD lista, tMatricula matrProcurada)
{
   for (int i = 0; i < lista->nElementos; ++i) {
      if (!strcmp(lista->elementos[i].matr, matrProcurada)) {
         return i; /* Elemento foi encontrado */
      }
   }

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * AcrescentaListaIdxD(): Acrescenta um novo elemento ao final
 *                       de uma lista sem ordenao
 *
 * Parmetros:
 *      lista (sada) - lista na qual ser feita o acrscimo
 *      elemento (entrada) - valor do elemento que ser
 *                           acrescentado
 *
 * Retorno: Nada
 *
 ****/
void AcrescentaListaIdxD(tListaIdxD lista, const tElemento *elemento)
{
   tElemento *novoArray; /* Ponteiro para o array */
                         /* redimensionado        */
   int        nBytesArray; /* Nmero de bytes do  */
                           /* array aps insero */

      /*                                                */
      /* Nessa abordagem, o tamanho da lista coincide   */
      /* com o tamanho do array, de modo que nunca h   */
      /* espao desocupado no array e de modo que       */
      /* redimensionamento sempre se faz necessrio     */
      /* quando ocorre insero ou remoo de elementos */
      /*                                                */

      /* Calcula o novo tamanho do array em bytes */
   nBytesArray = (lista->nElementos + 1)*sizeof(tElemento);

      /* Tenta redimensionar o array */
   novoArray = realloc(lista->elementos, nBytesArray);

      /* Garante que houve realocao de memria */
   ASSEGURA(novoArray, "Nao foi possivel redimensionar o array");

      /* O ponteiro que apontava para o incio */
      /* do array pode no ser mais vlido.    */
      /* Portanto,  preciso atualiz-lo.      */
   lista->elementos = novoArray;

      /* Acrescenta o novo elemento ao final da lista */
   lista->elementos[lista->nElementos] = *elemento;

   ++lista->nElementos; /* O tamanho da lista aumentou */
}

/****
 *
 * AlteraElementoListaIdxD(): Altera o valor de um elemento que se
 *                   encontra num determinada posio de
 *                   uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada e sada) - lista na qual o elemento
 *                                 se encontra
 *       elemento (entrada) - valor do elemento que ser
 *                            acrescentado
 *
 * Retorno: Nada
 *
 ****/
void AlteraElementoListaIdxD(tListaIdxD lista, int indice,
                    const tElemento *novoValor)
{
         /* Garante que o ndice  vlido */
   ASSEGURA(indice >= 0 && indice < lista->nElementos,
            "Posicao de alteracao inexistente");

      /* Altera o valor do elemento */
   lista->elementos[indice] = *novoValor;
}
