/****
 *
 * Ttulo: Lista Ordenada de Alunos
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Gerencia uma lista de compras permitindo
 *            desfazer e refazer operaes
 *
 * Entrada: Itens (strings) e opes escolhidas pelo usurio
 *
 * Sada: A lista criada pelo usurio
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Funes de Entrada/Sada */
#include <string.h>   /* Processamento de strings */
#include <stdlib.h>   /* Funo strtod()          */
#include "Lista.h"    /* Interface do tipo lista  */
#include "Leitura.h"  /* Leitura via teclado      */
#include "ASSEGURA.h" /* Macro ASSEGURA()         */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo de uma linha no arquivo */
#define TAMANHO_LINHA 60

   /* Nome do arquivo binrio (diretrio corrente) */
#define NOME_ARQ_BIN "Tudor.bin"

/***************** Definies de Funes ******************/

/****
 *
 * ExibeElemento(): Exibe na tela um elemento de uma
 *                  lista ordenada
 *
 * Parmetros: *elemento (entrada) - elemento que ser exibido
 *
 * Retorno: Nada
 *
 ****/
static void ExibeElemento(const tElemento *elemento)
{
   printf("\n%s", elemento->nome);
   printf("\t%s", elemento->matr);
   printf("\t%3.1f", elemento->n1);
   printf("\t%3.1f", elemento->n2);
}

/****
 *
 * ExibeUmElemento(): Exibe na tela um elemento de uma
 *                    lista ordenada
 *
 * Parmetros: *elemento (entrada) - elemento que ser exibido
 *
 * Retorno: Nada
 *
 ****/
static void ExibeUmElemento(const tElemento *elemento)
{
   printf("\n>>> %s", elemento->nome);
   printf("\t%s", elemento->matr);
   printf("\t%3.1f", elemento->n1);
   printf("\t%3.1f\n", elemento->n2);
}

/****
 *
 * ExibeLista(): Apresenta na tela, em forma de tabela, os
 *               elementos de uma lista de estruturas do tipo
 *               tAluno armazenada num array
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeLista(tListaIdxD lista)
{
   tElemento elemento;
   int       tamanho = ComprimentoListaIdxD(lista);

      /* Se a lista estiver vazia, no h o que exibir */
   if (!tamanho) {
      printf("\n*** Lista Vazia *** \n");
      return;
   }

      /* Cabealho de apresentao na tela */
   printf( "\n   Nome   \tMatr\tN1\tN2"
           "\n   ====   \t====\t==\t==\n");

      /* Apresenta cada estrutura numa linha separada */
   for (int i = 0; i < tamanho; ++i) {
      elemento = ObtemElementoListaIdxD(lista, i);
      ExibeElemento(&elemento);
   }

   putchar('\n'); /* Embelezamento apenas */
}

/****
 *
 * LeArquivo(): L arquivo de dados (binrio) e o armazena
 *              numa lista sem ordenao
 *
 * Parmetros:
 *      arq (entrada) - nome do arquivo
 *     *lista (sada) - lista que conter os registros lidos
 *
 * Retorno: Nada
 *
 ****/
void LeArquivo( const char *arq, tListaIdxD lista )
{
   FILE      *stream;
   tElemento  umElemento;

      /* Abre arquivo em formato binrio para leitura */
   stream = fopen(arq, "rb");

      /* Se o arquivo no puder ser aberto, */
      /* nada mais pode ser feito           */
   ASSEGURA(stream, "Arquivo nao pode ser aberto");

      /* L cada registro do arquivo */
      /* e acrescenta-o na lista     */
   while (1) {
         /* L um registro no arquivo */
      fread( &umElemento, sizeof(umElemento), 1, stream);

         /* Verifica se ocorreu erro ou o */
         /* final do arquivo foi atingido */
      if (ferror(stream) || feof(stream)) {
         break;
      }

         /* Acrescenta o registro lido ao final da lista */
      AcrescentaListaIdxD(lista, &umElemento);
   }

   ASSEGURA(!ferror(stream), "Erro de leitura de arquivo ");

      /* Processamento terminado. Fecha o arquivo. */
   fclose(stream);
}

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes para o usurio
 *
 * Parmetros:
 *      opcoes[] (entrada) - array de strings contendo as
 *                           opes do usurio
 *      nOpcoes (entrada) - o nmero de opes
 *
 * Retorno: Nada
 *
 ****/
static void ApresentaMenu(const char *opcoes[], int nOpcoes)
{
   int i;

   printf("\nEscolha uma das opcoes a seguir:\n");

   for (i = 0; i < nOpcoes; ++i)
      printf("\n\t(%d) %s", i + 1, opcoes[i]);

   printf("\n\n\t>>> ");
}

/****
 *
 * LeDadosAluno(): L os dados de um aluno
 *
 * Parmetros: *aluno (sada) - estrutura que receber os
 *                              dados introduzidos pelo
 *                              usurio
 *             fMatr (entrada) - indica se o campo 'matr'
 *                               ser lido ou no.
 *
 * Retorno: O parmetro
 *
 * Observao: O parmetro 'fMatr'  uma flag que
 *             determina se a matrcula deve ser lida
 *             ('fMatr' diferente de zero) ou no
 *             ('fMatr' igual a zero). A justificativa
 *             para o uso deste parmetro  que, algumas
 *             vezes, quando esta funo  chamada, a
 *             matrcula j ter sido lida. Esta alternativa
 *              melhor do que solicitar novamente a
 *             matrcula ao usurio (inconveniente) ou
 *             escrever duas funes que diferem apenas
 *             pelo fato de solicitarem matrcula ou no.
 *
 ****/
tAluno *LeDadosAluno(tAluno *aluno, int fMatr)
{
      /* L o nome */
   LeNome(aluno->nome, MAX_NOME + 1);

      /* Se indicado pelo parmetro */
      /* 'fMatr', l a matrcula    */
   if (fMatr) {
      LeMatricula(aluno->matr, TAM_MATR + 1);
   }

      /* L a primeira nota */
   printf("\n\t>>> 1a. nota:\n\t    > ");
   aluno->n1 = LeNota();

      /* L a segunda nota */
   printf("\n\t>>> 2a. nota:\n\t    > ");
   aluno->n2 = LeNota();

   return aluno;
}

/****
 *
 * AtualizaArquivoBin(): Escreve cada estrutura do tipo tAluno
 *                       de uma lista num arquivo binrio
 *
 * Parmetros:
 *      nomeArquivo (entrada) - nome do arquivo binrio no qual
 *                              as estruturas sero escritas
 *      lista (entrada) - lista contendo as estruturas que sero
 *                        escritas no arquivo
 *
 * Retorno: 0, se ocorrer zero erro. 1, em caso contrrio.
 *
 ****/
void AtualizaArquivoBin( const char *nomeArquivo, tListaIdxD lista )
{
   FILE     *stream; /* Stream associado ao arquivo  */
                   /* no qual ser feita a escrita */
   int       tamLista = ComprimentoListaIdxD(lista); /* Tamanho da lista */
   tElemento elemento;

      /* Se a lista estiver vazia, no */
      /* h o que escrever no arquivo  */
   if (!tamLista) {
      return;
   }

      /* Abre arquivo em formato binrio para escrita */
   stream = fopen(nomeArquivo, "wb");

      /* Se o arquivo no foi aberto, */
      /* nada mais pode ser feito     */
   if (!stream) {
      printf("\nArquivo %s nao pode ser aberto\n", nomeArquivo);
      exit(1); /* Arquivo no foi aberto */
   }

      /* Escreve cada elemento da lista no arquivo */
   for (int i = 0; i < tamLista; ++i) {
      elemento = ObtemElementoListaIdxD(lista, i);
      fwrite(&elemento, sizeof(elemento), 1, stream);

      ASSEGURA( !ferror(stream),
                "Erro de escrita no arquivo binario" );
   }

      /* Processamento terminado. Fecha o arquivo. */
   fclose(stream);
}

/****
 *
 * main(): Testa as operaes sobre listas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main (void)
{
   tListaIdxD      lista;
   tAluno      aluno;
   const char *opcoes[] = { "Acrescenta um aluno",
                            "Remove um aluno",
                            "Consulta dados de aluno",
                            "Altera dados de aluno",
                            "Exibe a turma na tela",
                            "Encerra o programa"
                          };
   int         indice,
               op,
               nOpcoes = sizeof(opcoes)/sizeof(opcoes[0]);
   char        umaMatr[TAM_MATR + 1 ];

   lista = CriaListaIdxD();

      /* Armazena contedo do arquivo   */
      /* de dados e o armazena na lista */
   LeArquivo(NOME_ARQ_BIN, lista);

   while (1) {
      ApresentaMenu(opcoes, nOpcoes);

      op = LeOpcao("123456");

      if (op == '6') { /* Encerra o programa */
            /* Atualiza arquivo binrio */
         AtualizaArquivoBin(NOME_ARQ_BIN, lista);

         break; /* Sada do lao */
      }

      switch (op) {
         case '1': /* Acrescenta um aluno */
               /* L dados do novo aluno */
               /* incluindo matrcula    */
            LeDadosAluno(&aluno, 1);

            AcrescentaListaIdxD(lista, &aluno);
            printf("\n>>> Acrescimo bem sucedido\n");

            break;

         case '2': /* Remove um aluno */
               /* L a matrcula introduzida pelo usurio */
            LeMatricula(umaMatr, TAM_MATR + 1);

            indice = BuscaSequencialListaIdxD(lista, umaMatr);

            if (indice >= 0) {
               (void) RemoveListaIdxD(lista, indice);
               printf("\n>>> Remocao bem sucedida\n");
            } else {
               printf("\n>>> Matricula nao encontrada\n");
            }

            break;

         case '3': /* Consulta dados de um aluno */
            LeMatricula(umaMatr, TAM_MATR + 1);
            indice = BuscaSequencialListaIdxD(lista, umaMatr);

            if (indice < 0) {
               printf("\n>>> Aluno nao foi encontrado\n");
            } else {
               aluno = ObtemElementoListaIdxD(lista, indice);
               ExibeUmElemento(&aluno);
            }

            break;

         case '4': /* Altera dados de um aluno */
               /* L a matrcula introduzida pelo usurio */
            LeMatricula(aluno.matr, TAM_MATR + 1);

            indice = BuscaSequencialListaIdxD(lista, umaMatr);

            if (indice < 0) {
               printf("\n>>> Matricula nao encontrada\n");
               break;
            }

               /* L dados do novo aluno sem incluir */
               /* a matrcula que j foi lida        */
            LeDadosAluno(&aluno, 0);

               /* Efetua a alterao */
            AlteraElementoListaIdxD(lista, indice, &aluno);

            break;

         case '5': /* Apresenta a turma na tela */
            ExibeLista(lista);
            break;

         default: /* O programa no deve chegar at aqui */
            printf("\nEste programa contem um erro\n");
            return 1;
      }


   }

         /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

return 0;
}

/******************* Exemplo de Execuo ******************

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 5

   Nome         Matr    N1      N2
   ====         ====    ==      ==

Ana Bolena      1012    7.8     8.0
Ana de Cleves   1022    4.5     6.0
Catarina Aragon 1014    5.5     6.5
Catarina Howard 1340    6.0     7.7
Catarina Parr   1440    4.0     6.0
Henrique VIII   1029    9.5     9.0
Joana Seymour   1017    7.7     8.7

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 2

Digite uma matricula com exatamente 4 digitos:
        > 1014

>>> Remocao bem sucedida

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 1

Digite o nome (max = 20 letras):
        > Mary Boleyn

Digite uma matricula com exatamente 4 digitos:
        > 0110

        >>> 1a. nota:
            > 7.2

        >>> 2a. nota:
            > 6.8

>>> Acrescimo bem sucedido

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 3

>>> Deseja consultar por nome (N) ou por matricula (M)? n

Digite o nome (max = 20 letras):
        > Catarina Parr

Catarina Parr   1440    4.0     6.0

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 3

>>> Deseja consultar por nome (N) ou por matricula (M)? m

Digite uma matricula com exatamente 4 digitos:
        > 1440

Catarina Parr   1440    4.0     6.0

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 5

   Nome         Matr    N1      N2
   ====         ====    ==      ==

Ana Bolena      1012    7.8     8.0
Ana de Cleves   1022    4.5     6.0
Catarina Howard 1340    6.0     7.7
Catarina Parr   1440    4.0     6.0
Henrique VIII   1029    9.5     9.0
Joana Seymour   1017    7.7     8.7
Mary Boleyn     0110    7.2     6.8

Escolha uma das opcoes a seguir:

        (1) Acrescenta um aluno
        (2) Remove um aluno
        (3) Consulta dados de aluno
        (4) Altera dados de aluno
        (5) Exibe a turma na tela
        (6) Encerra o programa

        >>> 6

        >>> Obrigado por usar este programa.

**********************************************************/
