/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada         */
#include <string.h>   /* Funes strXXX()        */
#include <stdlib.h>   /* malloc()                */
#include "ASSEGURA.h" /* Macro ASSEGURA()        */
#include "Lista.h"    /* Interface do tipo lista */

/***************** Constantes Simblicas ******************/

   /* Nmero de elementos que sero acrescentados ao */
   /* array a cada alocao ou realocao de memria */
#define ACRESCIMO 10

/****************** Complemento de Tipo *******************/

struct rotListaIdxD2 {
   tElemento *elementos; /* Ponteiro para o array que     */
                         /* conter os elementos da lista */
   int        nElementos;   /* Nmero de elementos da lista */
   int        tamanhoArray; /* Nmero de elementos do array */
};

/***************** Definies de Funes ******************/

/****
 *
 * CriaListaIdxD2(): Cria uma lista
 *
 * Parmetros: No h
 *
 * Retorno: Endereo da lista criada
 *
 ****/
tListaIdxD2 CriaListaIdxD2(void)
{
   tListaIdxD2 umaLista;

      /* Aloca espao para a estrutura que armazena a lista */
   umaLista = malloc(sizeof(struct rotListaIdxD2));

      /* Garante que a alocao realmente ocorreu */
   ASSEGURA(umaLista, "Nao foi possivel alocar lista");

      /* Inicialmente a lista est vazia */
   umaLista->nElementos = 0;
   umaLista->elementos = NULL;

      /* Aloca previamente um array */
   umaLista->elementos = calloc(ACRESCIMO, sizeof(tElemento));

      /* Garante que o array foi realmente alocado */
   ASSEGURA( umaLista->elementos, "Nao foi possivel alocar "
             "array para conter elementos da lista" );

      /* Atualiza o campo que armazena o tamanho do array */
   umaLista->tamanhoArray = ACRESCIMO;

   return umaLista;
}

/****
 *
 * DestroiListaIdxD2(): Libera o espao ocupado pela lista
 *
 * Parmetros: lista (entrada) - a lista
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaIdxD2(tListaIdxD2 lista)
{
      /* Libera o array que contm os elementos */
   free(lista->elementos);

      /* Libera a prpria lista */
   free(lista);
}

/****
 *
 * ComprimentoListaIdxD2(): Calcula o comprimento de uma lista
 *
 * Parmetros: *lista (entrada) - a lista cujo comprimento
 *                                ser calculado
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
int ComprimentoListaIdxD2(tListaIdxD2 lista)
{
   return lista->nElementos; /* No h o que calcular */
}

/****
 *
 * ObtemElementoListaIdxD2(): Obtm o elemento de uma lista que se
 *                  encontra numa dada posio
 *
 * Parmetros:
 *     *lista (entrada) - a lista
 *      indice (entrada) - o elemento
 *
 * Retorno: o nmero de elementos na lista
 *
 ****/
tElemento ObtemElementoListaIdxD2(tListaIdxD2 lista, int indice)
{
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Elemento inexistente" );

   return lista->elementos[indice];
}

/****
 *
 * InsereEmOrdemIdxD2(): Insere um novo elemento numa lista de
 *                  modo a preservar a ordenao da lista
 *
 * Parmetros:
 *      lista (entrada e sada) - lista na qual ser feita
 *                                a insero
 *      elemento (entrada) - elemento que ser inserido
 *
 * Retorno: Nada
 *
 ****/
void InsereEmOrdemIdxD2(tListaIdxD2 lista, const tElemento *elemento)
{
   int        posicao; /* Posio de insero do elemento */
   tElemento *novoArray; /* Ponteiro para o array */
                         /* redimensionado        */

      /* Se o array estiver repleto, tenta redimension-lo */
   if (lista->nElementos >= lista->tamanhoArray) {
         /* Calcula o novo tamanho do array */
      lista->tamanhoArray += ACRESCIMO;

         /* Tenta redimensionar o array */
      novoArray = realloc(lista->elementos,
                          lista->tamanhoArray*sizeof(tElemento));

         /* Checa se houve realocao de memria */
      ASSEGURA(novoArray, "Nao houve redimensionamento");

         /* O ponteiro que apontava para o incio */
         /* do array pode no ser mais vlido.    */
         /* Portanto,  preciso atualiz-lo.      */
      lista->elementos = novoArray;
   }

      /* Se a lista estiver vazia, o elemento  */
      /* inserido na primeira posio do array  */
   if (EstaVaziaListaIdxD2(lista)) {
         /* O elemento ser o primeiro da lista */
      lista->elementos[0] = *elemento;

      ++lista->nElementos; /* A lista cresceu */

      return;
   }

      /* Encontra a posio no array onde */
      /* o elemento ser inserido         */
   for (posicao = 0; posicao < lista->nElementos; ++posicao) {
      if ( strcmp(lista->elementos[posicao].nome,
                  elemento->nome) > 0 ) {
         break; /* Posio de insero encontrada */
      }
   }

      /* Abre espao para o novo elemento */
   for (int i = lista->nElementos - 1; i >= posicao; --i) {
         /* Move cada elemento uma posio       */
         /* adiante a partir do elemento que ora */
         /* se encontra na posio de insero   */
      lista->elementos[i + 1] = lista->elementos[i];
   }

      /* Insere o novo elemento */
   lista->elementos[posicao] = *elemento;

   ++lista->nElementos; /* A lista cresceu */
}

/****
 *
 * RemoveListaIdxD2(): Remove um elemento numa dada posio
 *                   de uma lista sem ordenao
 *
 * Parmetros:
 *      *lista (entrada/sada) - lista onde ser feita a
 *                               remoo
 *       indice (entrada) - posio do elemento que ser removido
 *
 * Retorno: O item removido
 *
 ****/
tElemento RemoveListaIdxD2(tListaIdxD2 lista, int indice)
{
   tElemento itemRemovido;

         /* Verifica se o ndice  vlido */
   ASSEGURA( indice >= 0 && indice < lista->nElementos,
             "Posicao de remocao inexistente" );

   itemRemovido = lista->elementos[indice];

      /* Remover um elemento significa mover cada   */
      /* elemento uma posio para trs a partir    */
      /* do sucessor do elemento que ser  removido */
   for (int i = indice; i < lista->nElementos - 1; i++)
      lista->elementos[i] = lista->elementos[i + 1];

   --lista->nElementos; /* O tamanho da lista diminuiu */

   return itemRemovido;
}

/****
 *
 * EstaVaziaListaIdxD2(): Verifica se uma lista sem ordenao est vazia
 *
 * Parmetros: *lista (entrada) - a lista que ser testada
 *
 * Retorno: 1, se a lista estiver vazia; 0, em caso contrario
 *
 ****/
int EstaVaziaListaIdxD2(tListaIdxD2 lista)
{
   return lista->nElementos == 0;
}

/****
 *
 * BuscaBinariaListaIdxD2(): Efetua uma busca binria numa
 *                 lista de elementos do tipo int
 *                 implementada usando array esttico
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       elemento (entrada) - valor do elemento que ser
 *                            procurado
 *
 * Retorno: ndice do primeiro elemento encontrado no array
 *          que casa com o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 * Observao: Na busca binria, quando um valor  encontrado
 *             e existe mais de uma ocorrncia desse valor no
 *             array, o ndice retornado no corresponde
 *             necessariamente ao menor ndice de elemento
 *             que contm o valor procurado.
 *
 ****/
int BuscaBinariaListaIdxD2(tListaIdxD2 lista, tNome nomeProcurado)
{
   int inf, /* Limite inferior da busca */
       sup, /* Limite superior da busca */
       meio, /* Meio do intervalo */
       comp; /* Resultado de uma comparao */

      /* limites inferior e superior iniciais */
   inf = 0;
   sup = lista->nElementos - 1;

      /* Efetua a busca binria */
   while (inf <= sup) {
         /* Calcula o meio do intervalo */
      meio = (inf + sup)/2;

         /* Compara o nome procurado com aquele do    */
         /* elemento que se encontra no meio da lista */
      comp = strcmp(nomeProcurado, lista->elementos[meio].nome);

         /* Verifica se o elemento se encontra */
         /* no meio do intervalo               */
      if (!comp) {
         return meio; /* Elemento encontrado */
      }

         /* Ajusta o intervalo de busca */
      if (comp < 0) {
         sup = meio - 1; /* Procura na metade inferior */
      } else {
         inf = meio + 1; /* Procura na metade superior */
      }
   }

   return -1; /* Elemento no foi encontrado */
}

/****
 *
 * BuscaSequencialListaIdxD2(): Efetua uma busca sequencial numa
 *                    lista de elementos do tipo tAluno
 *                   implementada usando array esttico
 *
 * Parmetros:
 *      *lista (entrada) - lista na qual ser efetuada a busca
 *       matrProcurada (entrada) - valor da matrcula do elemento
 *                                  que ser procurado
 *
 * Retorno: ndice do primeiro elemento no array que casa com
 *          o valor recebido como parmetro.
 *          Um valor negativo, se o valor no for encontrado.
 *
 ****/
int BuscaSequencialListaIdxD2(tListaIdxD2 lista, tMatricula matrProcurada)
{
   for (int i = 0; i < lista->nElementos; ++i) {
      if (!strcmp(lista->elementos[i].matr, matrProcurada)) {
         return i; /* Elemento foi encontrado */
      }
   }

   return -1; /* Elemento no foi encontrado */
}
