/****
 *
 * Ttulo: A Urupema de Eratstenes
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/09/2012
 * ltima modificao: 16/09/2012
 *
 * Descrio: Solicita um inteiro positivo ao usurio e apresenta
 *            os nmeros primos menores do que ou iguais ao valor
 *            introduzido usando o Crivo de Eratstenes.
 *
 * Entrada: O referido valor inteiro
 *
 * Sada: Os nmeros primos menores do que ou iguais
 *        ao referido valor
 *
 ****/

#include <stdio.h>   /* Entrada e sada   */
#include <stdlib.h>  /* Alocao dinmica */

   /* Quantidade de nmeros primos exibidos por linha */
#define PRIMOS_POR_LINHA  8

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeInteiro(): L um nmero inteiro via teclado
 *              e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero inteiro lido
 *
 ****/
int LeInteiro(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n",
             nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * Eratostenes(): Encontra todos os nmeros primos menores do que
 *                o valor especificado como parmetro e exibe-os
 *                na tela usando a tcnica do Crivo de Eratstenes
 *
 * Parmetros:
 *     n (entrada) - maior valor que se verificar se  primo
 *
 * Retorno: 1, se ocorrer erro. 0, se no ocorrer erro.
 *
 ****/
int Eratostenes(int n)
{
   int *ar,      /* Array usado como peneira */
        primo,   /* Armazena um nmero primo */
        tamanho, /* Tamanho da peneira       */
        i, j;

      /* Verifica se valor recebido como */
      /* parmetro  inconveniente       */
   if (n <= 0) {
      return 1; /* Valor deveria ser pelo menos igual a 1 */
   }

      /* O menor nmero primo  2. Logo, a peneira dever */
      /* armazenar os nmeros 2, 3, ..., n . Assim, o     */
      /* tamanho da peneira (array) deve ser n - 1.       */
   tamanho = n - 1;

      /* Tenta alocar o array que servir de coador */
   ar = calloc(tamanho, sizeof(int));

      /* Se no houve alocao, no  possvel prosseguir */
   if (!ar) {
      return 1; /* Array no foi alocado */
   }

      /* Inicia os elementos do array com valores entre 2 e n */
   for(i = 0; i < tamanho; i++) {
      ar[i] = i + 2; /* 2  o menor primo */
   }

      /* Seleciona um nmero primo a partir do primeiro   */
      /* elemento do array e atribui zero a todos os      */
      /* demais elementos do array que so seus mltiplos */
   for(i = 0; i < tamanho; i++) {
         /* O prximo primo considerado  o elemento  */
         /* imediato do array que  diferente de zero */
      if(ar[i]) {
         primo = ar[i]; /* Se no  zero,  primo */
      } else {
            /* O restante do lao deve ser saltado, pois */
            /* no foi encontrado um novo nmero primo   */
         continue;
      }

            /*                                           */
            /* Atribui zero a cada elemento do array que */
            /*  mltiplo do ltimo primo encontrado     */
            /*                                           */

         /* O primeiro mltiplo de um nmero primo  seu dobro */
      j = 2*primo;

      while(j <= n) {
            /* Atribui zero ao elemento que  igual ao */
            /* ltimo mltiplo de 'primo'. Note que o  */
            /* elemento cujo valor  j est armazenado */
            /* no elemento de  ndice j - 2.           */
         ar[j - 2] = 0;

         j = j + primo; /* Obtm o prximo mltiplo */
      }
   }

      /*                                                  */
      /* Neste ponto, todos os elementos do array que no */
      /* so primos tm valor zero; i.e., os elementos do */
      /* array foram joeirados na Joeira de Eratstenes   */
      /*                                                  */

   printf("\n\t  *** Primos entre 2 e %d ***\n", n);

      /* Apresenta na tela os elementos que no  */
      /* so nulos; i.e., aqueles que so primos */
   for(i = 0, j = 0; i < tamanho; ++i) {
      if(ar[i]) { /* Se no  zero,  primo */
            /* Quebra linha quando o nmero de primos */
            /* exibidos atinge um valor especificado  */
         if (!(j%PRIMOS_POR_LINHA)) {
            printf("\n");
         }

         printf("%4d  ", ar[i]); /* Exibe mais um nmero primo */

         ++j; /* Mais um primo foi exibido */
      }
   }

   putchar('\n'); /* Embelezamento */

   free(ar); /* Libera espao ocupado pelo array */

   return 0; /* A urupema funcionou */
}

/****
 *
 * main(): Solicita um nmero inteiro positivo ao usurio e
 *         apresenta os nmeros primos menores do que ou iguais
 *         ao valor introduzido usando o Crivo de Eratstenes
 *
 * Parmetros: Nenhum
 *
 * Retorno: 0, se no ocorrer nenhum erro; 1, em caso contrrio.
 *
 ****/
int main(void)
{
   int numero;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa encontra os numeros primos que "
           "se\n\t>>> encontram entre 2 e o valor inteiro "
           "positivo que\n\t>>> voce introduzir.\n" );

      /* L o nmero */
   printf("\n\t>>> Digite o numero: ");
   numero = LeInteiro();

      /* Verifica se valor introduzido  vlido e, se for */
      /* o caso, apresenta os nmeros primos peneirados   */
   if (numero > 1) {
      if (Eratostenes(numero)) {
         printf("\n\t>>> Impossivel encontrar primos\n");
         return 1; /* Peneira de Eratstenes est rasgada */
      }
   } else {
      printf("\n\t>>> O numero deve ser maior do que 1\n");
      return 1; /* Usurio no sabe o que  nmero primo */
   }

   printf( "\n\t>>> Obrigado por usar este programa e visite"
           "\n\t>>> o Nordeste para conhecer uma urupema.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa encontra os numeros primos que se
        >>> encontram entre 2 e o valor inteiro positivo que
        >>> voce introduzir.

        >>> Digite o numero: 200

          *** Primos entre 2 e 200 ***

   2     3     5     7    11    13    17    19
  23    29    31    37    41    43    47    53
  59    61    67    71    73    79    83    89
  97   101   103   107   109   113   127   131
 137   139   149   151   157   163   167   173
 179   181   191   193   197   199

        >>> Obrigado por usar este programa e visite
        >>> o Nordeste para conhecer uma urupema.

**********************************************************/
