/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada   */
#include <stdlib.h>  /* Alocao dinmica */
#include "ASSEGURA.h" /* Macro ASSEGURA */
#include "Lista.h"    /* Interface deste arquivo */

/***************** Definies de Funes ******************/

/****
 *
 * IniciaListaSE(): Inicia uma lista simplesmente encadeada
 *
 * Parmetros: *lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
void IniciaListaSE(tListaSE *lista)
{
   *lista = NULL;
}

/****
 *
 * ComprimentoListaSE(): Calcula o nmero de ns de uma lista
 *                       simplesmente encadeada
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para o primeiro n da lista
 *                       cujo comprimento ser calculado
 *
 * Retorno: O nmero de ns da referida lista
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
int ComprimentoListaSE(tListaSE lista)
{
   int tamanho = 0; /* Armazena o nmero de ns da lista */

      /* Acessa cada n da lista e conta */
      /* quantos ns so acessados       */
   while (lista) {
      ++tamanho; /* Mais um n foi encontrado */
      lista = lista->proximo; /* Passa para o prximo n */
   }

   return tamanho;
}

/****
 *
 * InsereEmOrdemLSE(): Insere um novo n no incio de uma
 *                     lista simplesmente encadeada ordenada
 *
 * Parmetros:
 *    *lista (entrada/sada) - ponteiro para a lista na qual
 *                             ser feita a insero
 *     conteudo (entrada) - contedo do n que ser inserido
 *
 * Retorno: Nada
 *
 * Observao: Como a lista no obedece nenhuma ordem, pode-se
 *             inserir um n em qualquer posio da lista.
 *             Escolheu-se insero no incio da lista porque
 *              mais simples e eficiente.
 *
 ****/
void InsereEmOrdemLSE(tListaSE *lista, tConteudo conteudo)
{
   tNoSE    *ptrNovoNo; /* Apontar para o novo n alocado */
   tListaSE  p = *lista, /* p aponta para o n corrente     */
             q = NULL;   /* q aponta para o n anterior a p */

      /* Tenta alocar um novo n */
   ASSEGURA( ptrNovoNo = malloc(sizeof(tNoSE)),
             "Nao foi possivel alocar no'" );

      /* Armazena no novo n os dados recebidos como parmetro */
   ptrNovoNo->conteudo = conteudo;

      /* Se a lista estiver vazia, basta adicionar */
      /* o n ao incio da lista e retornar        */
   if (!*lista) {
      *lista = ptrNovoNo; /* Incio da lista apontar */
                          /* para novo n             */
      (*lista)->proximo = NULL; /* O novo n  o nico da lista */
      return;
   }

         /*************************************************/
         /* A lista tem pelo menos um n. Nesse caso, a   */
         /* posio de insero  aquela do primeiro n   */
         /* cujo contedo seja maior do que aquele do n  */
         /* a ser inserido. Se tal n no for encontrado, */
         /* o novo n ser o ltimo da lista.             */
         /*************************************************/

      /* Procura o local de insero */
   while ( p && p->conteudo <= conteudo ) {
      q = p;    /* q passa a apontar para o n corrente */
      p = p->proximo; /* p passa a apontar para o prximo n */
   }

      /**********************************************************/
      /* Neste instante, p aponta para o local da insero e q  */
      /* aponta para o n imediatamente antes desse local. Se   */
      /* p for NULL, o novo n ser o ltimo da lista, mas isso */
      /* no constitui um caso especial a ser tratado  parte.  */
      /**********************************************************/

      /* O novo n apontar para o n apontado por p */
   ptrNovoNo->proximo = p;

      /* Verifica se p aponta para o primeiro n da lista pois */
      /* insero no incio deve ser tratada separadamente     */
   if (p == *lista) { /* Insero ser no incio da lista */
      *lista = ptrNovoNo;/* Insero no incio  especial */
   } else {
         /* O n anterior quele apontado */
         /* por p apontar para o novo n */
      q->proximo = ptrNovoNo;
   }
}

/****
 *
 * RemoveListaSE(): Remove de uma lista simplesmente encadeada
 *                um n que contm um determinado valor
 *                especificado como parmetro
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro que representa
 *                             a lista na qual ser feita a remoo
 *     conteudo (entrada) - contedo do n a ser removido
 *
 * Retorno: 0, se a remoo for bem sucedida, ou 1, se o n a
 *          ser removido no for encontrado.
 *
 ****/
int RemoveListaSE(tListaSE *lista, tConteudo conteudo)
{
   tListaSE p = *lista, /* p aponta para o n corrente     */
            q = NULL;   /* q aponta para o n anterior a p */

      /*                                                     */
      /* O lao while procura um n que cujo contedo case   */
      /* com o parmetro 'conteudo' e tem duas condies de  */
      /* parada: (1) todos os ns so acessados sem que o    */
      /* referido n tenha sido encontrado (neste  caso, p   */
      /* assume NULL); (2) o n que contm o valor procurado */
      /*  encontrado.                                       */
      /*                                                     */
   while (p && p->conteudo != conteudo) {
      q = p; /* q passa a apontar para o n corrente */
      p = p->proximo; /* p passa a apontar para o prximo n */
   }

      /* Se p for NULL, a lista foi totalmente examinada   */
      /* sem que o n a ser removido tenha sido encontrado */
   if (!p) {
      return 1; /* Tentativa de remoo de n inexistente */
   }

      /****************************************************/
      /* Neste ponto, p aponta para o n a ser removido e */
      /* q aponta para o n imediatamente anterior a ele  */
      /****************************************************/

      /* Verifica se p aponta para o primeiro */
      /* n da lista porque remoo no incio */
      /* deve ser tratada separadamente       */
   if (p == *lista) { /* Remoo ser no incio da lista */
      (*lista) = p->proximo; /* Altera incio da lista */
   } else {/* N a ser removido NO  o primeiro da lista */
      q->proximo = p->proximo; /* Desvia o n anterior */
                               /* do n a ser removido */
   }

      /* Neste ponto, o n desejado no faz mais parte da */
      /* lista, mas ele ainda ocupa espao em memria.    */
      /* Portanto,  necessrio liberar este espao.      */
   free(p); /* Libera o n removido */

   return 0;
}

/****
 *
 * EstaVaziaListaSE(): Verifica se uma lista simplesmente
 *                 encadeada est vazia
 *
 * Parmetros:
 *     lista (entrada) - a lista que ser verificada
 *
 * Retorno: 1, quando a lista est vazia, ou 0, em caso contrrio
 *
 ****/
int EstaVaziaListaSE(tListaSE lista)
{
   return lista == NULL;
}

/****
 *
 * BuscaListaSEOrd(): Retorna o endereo do contedo efetivo do
 *                    n que possui um determinado contedo numa
 *                    lista simplesmente encadeada ordenada
 *
 * Parmetros:
 *      lista (entrada) - lista que ser pesquisada
 *      valor (entrada) - contedo do n procurado
 *
 * Retorno: Endereo dos dados que contm o referido contedo,
 *          se ele for encontrado. Caso contrrio, NULL.
 *
 ****/
tConteudo *BuscaListaSEOrd(tListaSE lista, tConteudo valor)
{
      /* Enquanto o ponteiro 'lista' no assume NULL   */
      /* ou o campo 'conteudo' de um n no  maior    */
      /* do que o parmetro 'valor', a busca prossegue */
   while (lista && lista->conteudo < valor) {
      lista = lista->proximo;
   }

      /* Se 'lista' assume NULL  porque o contedo     */
      /* especificado como parmetro no foi encontrado */
   if(!lista) {
      return NULL; /* N no foi encontrado */
   }

      /* Verifica se o n foi encontrado */
   if (lista->conteudo == valor) {
      return &lista->conteudo;
   } else {
      return NULL; /* N no foi encontrado */
   }
}

/****
 *
 * DestroiListaSE(): Libera o espao ocupado pelos ns de
 *                   uma lista encadeada, tornando-a vazia
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a
 *                             lista simplesmente encadeada
 *                             que ser destruda
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaSE(tListaSE *lista)
{
   tListaSE p; /* Aponta para o prximo n a ser liberado */

   if (!*lista) { /* Verifica se a lista est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *lista; /* Faz p apontar para o incio da lista */

      /* Visita cada n da lista liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /*o incio da lista seja alterado. Afinal, este    */
         /* ponteiro ser mesmo anulado.                    */
      *lista = (*lista)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *lista    */
      p = *lista;
   } while (p);

      /* Neste ponto, a lista ficou vazia, pois o  */
      /* ltimo valor assumido por *lista foi NULL */
}

/****
 *
 * ProximoListaSE(): Encontra o prximo n de uma lista
 *                   simplesmente encadeada
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *                       simplesmente encadeada
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *ProximoListaSE(tListaSE lista)
{
   static tNoSE *proximoNo = NULL;

   if (lista) {
      if (!proximoNo) {
         proximoNo = lista;
      } else {
         proximoNo = proximoNo->proximo;
      }
      return &proximoNo->conteudo;
   } else {
      return NULL;
   }
}

