/****
 *
 * Ttulo: Lista Esttica Ordenada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada   */
#include <stdlib.h>  /* Alocao dinmica */
#include "ASSEGURA.h" /* Macro ASSEGURA */
#include "Lista.h"    /* Interface deste arquivo */

/***************** Definies de Funes ******************/

/****
 *
 * IniciaListaSEC(): Inicia uma lista simplesmente encadeada
 *
 * Parmetros: *lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
void IniciaListaSEC(tListaSEC *lista)
{
   *lista = NULL;
}

/****
 *
 * ComprimentoListaSEC(): Calcula o nmero de ns de uma lista
 *                        simplesmente encadeada circular
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para o primeiro n da lista
 *                       cujo comprimento ser calculado
 *
 * Retorno: O nmero de ns da referida lista
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
int ComprimentoListaSEC(tListaSEC lista)
{
   int       tamanho = 0; /* Armazenar o tamanho da lista */
   tListaSEC inicio = lista; /* Guarda o incio da lista */

   if (!lista) /* A lista est vazia */
      return 0;

      /* A lista possui pelo menos um n. */
      /* Acessa cada n da lista e conta  */
      /* quantos ns so acessados.       */
   do {
      lista = lista->proximo; /* Passa para o prximo n */
      ++tamanho; /* Mais um n foi encontrado */
   } while (lista != inicio);

   return tamanho;
}

/****
 *
 * EstaVaziaListaSEC(): Verifica se uma lista simplesmente
 *                  encadeada circular est vazia
 *
 * Parmetros:
 *     lista (entrada) - a lista que ser verificada
 *
 * Retorno: 1, quando a lista est vazia, ou 0, em caso contrrio
 *
 ****/
int EstaVaziaListaSEC(tListaSEC lista)
{
   return lista == NULL;
}

/****
 *
 * InsereEmOrdemLSEC(): Insere um novo n no incio de uma
 *                      lista simplesmente encadeada circular
 *                      ordenada
 *
 * Parmetros:
 *    *lista (entrada/sada) - ponteiro para a lista na qual
 *                             ser feita a insero
 *     conteudo (entrada) - contedo do n que ser inserido
 *
 * Retorno: Nada
 *
 ****/
void InsereEmOrdemLSEC(tListaSEC *lista, tConteudo conteudo)
{
   tNoSEC    *ptrNovoNo; /* Apontar para o novo n alocado */
   tListaSEC  p = *lista, /* p aponta para o n corrente     */
              q = NULL;   /* q aponta para o n anterior a p */

      /* Tenta alocar um novo n */
   ASSEGURA( ptrNovoNo = malloc(sizeof(tNoSEC)),
             "Nao foi possivel alocar no'" );

      /* Armazena no novo n os dados recebidos como parmetro */
   ptrNovoNo->conteudo = conteudo;

      /* Se a lista estiver vazia, basta adicionar */
      /* o n ao incio da lista e retornar        */
   if (!*lista) {
         /* O novo n  o nico da lista e,    */
         /* portanto, apontar para si prprio */
      ptrNovoNo->proximo = ptrNovoNo;

         /* O incio da lista apontar para novo n */
      *lista = ptrNovoNo;
      return;
   }

         /*************************************************/
         /* A lista tem pelo menos um n. Nesse caso, a   */
         /* posio de insero  aquela do primeiro n   */
         /* cujo contedo seja maior do que aquele do n  */
         /* a ser inserido. Se tal n no for encontrado, */
         /* o novo n ser o ltimo da lista.             */
         /*************************************************/

      /* Procura o local de insero */
   do {
      q = p;
      p = p->proximo; /* Passa para o prximo n */
   } while (p != *lista && p->conteudo <= conteudo);


      /**********************************************************/
      /* Neste instante, p aponta para o local da insero e q  */
      /* aponta para o n imediatamente antes desse local. Se   */
      /* p for NULL, o novo n ser o ltimo da lista, mas isso */
      /* no constitui um caso especial a ser tratado  parte.  */
      /**********************************************************/

      /* O novo n apontar para o n apontado por p */
   ptrNovoNo->proximo = p;

      /* O n para o qual q aponta apontar para o novo n */
   q->proximo = ptrNovoNo;

      /* Verifica se p aponta para o primeiro n da */
      /* lista pois insero no incio  especial   */
   if (conteudo <= (*lista)->conteudo) {
      *lista = ptrNovoNo;/* Insero  no incio */
   }
}

/****
 *
 * RemoveListaSEC(): Remove de uma lista simplesmente encadeada
 *                 circular um n que contm um determinado valor
 *                 especificado como parmetro
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro que representa
 *                             a lista na qual ser feita a remoo
 *     conteudo (entrada) - contedo do n a ser removido
 *
 * Retorno: 0, se a remoo for bem sucedida, ou 1, se o n a
 *          ser removido no for encontrado.
 *
 ****/
int RemoveListaSEC(tListaSEC *lista, tConteudo conteudo)
{
   tListaSEC p = *lista, /* p aponta para o n corrente     */
             q = NULL;   /* q aponta para o n anterior a p */

   if (!*lista) {
      return 1; /* A lista est vazia */
   }

      /* Se a lista s tiver um n, a */
      /* remoo desse n  especial  */
   if (p->conteudo == conteudo && !p->proximo) {
      *lista = NULL;

      free(p); /* Libera o n */

      return 0; /* Remoo foi OK */
   }

      /* Procura o n que ser removido */
   do {
      q = p;
      p = p->proximo;
   } while (p != *lista && p->conteudo != conteudo);

      /* Se p estiver apontando para o primeiro n da    */
      /* lista e o valor procurado no se encontra nesse */
      /* n, a lista foi totalmente examinada sem que o  */
      /* n a ser removido tenha sido encontrado         */
   if (p == *lista && p->conteudo != conteudo) {
      return 1; /* Tentativa de remoo de n inexistente */
   }

      /****************************************************/
      /* Neste ponto, p aponta para o n a ser removido e */
      /* q aponta para o n imediatamente anterior a ele  */
      /****************************************************/

      /* Desvia o n anterior do n a ser removido */
   q->proximo = p->proximo;

      /* Verifica se p aponta para o primeiro */
      /* n da lista porque remoo no incio */
      /* deve ser tratada separadamente       */
   if (p == *lista) { /* Remoo ser no incio da lista */
      (*lista) = p->proximo; /* Altera incio da lista */
   }

      /* Libera o n removido */
   free(p);

   return 0;
}

/****
 *
 * BuscaListaSEC(): Retorna o endereo do contedo efetivo do
 *                     n que possui um determinado contedo numa
 *                     lista simplesmente encadeada circular
 *
 * Parmetros:
 *      lista (entrada) - lista que ser pesquisada
 *      valor (entrada) - contedo do n procurado
 *
 * Retorno: Endereo dos dados que contm o referido contedo,
 *          se ele for encontrado. Caso contrrio, NULL.
 *
 ****/
tConteudo *BuscaListaSEC(tListaSEC lista, tConteudo valor)
{
   tListaSEC inicio;

      /* Se a lista estiver vazia, o  */
      /* elemento no ser encontrado */
   if (!lista) {
      return NULL;
   }

      /* Verifica se o valor se encontra no primeiro n */
   if (lista->conteudo == valor) {
      return &lista->conteudo;
   }

      /* Guarda o incio da lista */
   inicio = lista;

      /* Passa para o segundo n se ele existir */
   lista = lista->proximo;

      /* Enquanto o ponteiro 'lista' no assume NULL */
      /* ou o campo 'conteudo' de um n no casa com */
      /* o parmetro 'valor', a busca prossegue      */
   while (lista != inicio && lista->conteudo != valor) {
      lista = lista->proximo;
   }

      /* Se 'lista' estiver apontando para o incio da */
      /* lista, o valor procurado no foi encontrado   */
   if(lista == inicio) {
      return NULL; /* N no foi encontrado */
   }

      /* Certamente, o valor foi encontrado */
   return &lista->conteudo;
}

/****
 *
 * DestroiListaSEC(): Libera o espao ocupado pelos ns de
 *                    uma lista simplesmente encadeada
 *                    circular, tornando-a vazia
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a
 *                             lista simplesmente encadeada
 *                             que ser destruda
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaSEC(tListaSEC *lista)
{
   tListaSEC p, /* Aponta para o prximo n a ser liberado */
             inicio; /* Guarda o incio da lista */

   if (!*lista) { /* Verifica se a lista est vazia */
      return; /* Lista vazia no precisa ser destruda */
   }

   p = *lista; /* Faz p apontar para o incio da lista */

   inicio = *lista; /* Faz p apontar para o incio da lista */

      /* Visita cada n da lista liberando-o */
   do {
         /* Passa para o prximo n antes que o n corrente */
         /* seja destrudo. No importa que o ponteiro para */
         /* o incio da lista seja alterado. Afinal, este   */
         /* ponteiro ser mesmo anulado.                    */
      *lista = (*lista)->proximo;

      free(p); /* Libera o espao do n corrente */

         /* Faz p apontar para o prximo n, cujo */
         /* endereo est armazenado em *lista    */
      p = *lista;
   } while (p != inicio);
}

/****
 *
 * ProximoListaSEC(): Encontra o prximo n de uma lista
 *                    simplesmente encadeada circular
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *                       simplesmente encadeada circular
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *ProximoListaSEC(tListaSEC lista)
{
   static tNoSEC *proximoNo = NULL;

   if (lista) {
      if (!proximoNo) {
         proximoNo = lista;
      } else {
         proximoNo = proximoNo->proximo;

         if (proximoNo == lista) {
            proximoNo = NULL;
         }
      }
      return proximoNo ? &proximoNo->conteudo : NULL;
   } else {
      return NULL;
   }
}

