/****
 *
 * Ttulo: Lista Duplamente Encadeada Circular
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada   */
#include <stdlib.h>   /* Alocao dinmica */
#include "ASSEGURA.h" /* Macro ASSEGURA    */
#include "Lista.h"    /* Interface deste arquivo */

/***************** Definies de Funes ******************/

/****
 *
 * IniciaListaDEC(): Inicia uma lista duplamente encadeada
 *                   circular
 *
 * Parmetros: *lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
void IniciaListaDEC(tListaDEC *lista)
{
   *lista = NULL;
}

/****
 *
 * ComprimentoListaDEC(): Calcula o nmero de ns de uma
 *                        lista duplamente encadeada circular
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para o primeiro n da lista
 *                       cujo comprimento ser calculado
 *
 * Retorno: O nmero de ns da referida lista
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
int ComprimentoListaDEC(tListaDEC lista)
{
   int       tamanho = 0; /* Armazenar o tamanho da lista */
   tListaDEC inicio = lista; /* Guarda o incio da lista */

   if (!lista) /* A lista est vazia */
      return 0;

      /* A lista possui pelo menos um n. */
      /* Acessa cada n da lista e conta  */
      /* quantos ns so acessados.       */
   do {
      lista = lista->proximo; /* Passa para o prximo n */
      ++tamanho; /* Mais um n foi encontrado */
   } while (lista != inicio);

   return tamanho;
}

/****
 *
 * InsereEmOrdemLDEC(): Insere um novo n numa lista
 *                      duplamente encadeada circular
 *
 * Parmetros:
 *    *lista (entrada/sada) - ponteiro para a lista na qual
 *                             ser feita a insero
 *     conteudo (entrada) - contedo do n que ser inserido
 *
 * Retorno: Nada
 *
 ****/
void InsereEmOrdemLDEC(tListaDEC *lista, tConteudo conteudo)
{
   tNoListaDEC    *ptrNovo;
   tListaDEC  p; /* p apontar para o n antes do */
                 /* qual ser feita a insero    */

      /* Tenta alocar um novo n */
   ptrNovo = malloc(sizeof(tNoListaDEC));

      /* Se o n no foi alocado, esta    */
      /* funo no tem mais nada a fazer */
   ASSEGURA( ptrNovo, "Nao foi possivel alocar um no'" );

      /* Armazena o contedo recebido */
      /*  como parmetro no novo n   */
   ptrNovo->conteudo = conteudo;

      /* Se a lista estiver vazia, basta adicionar */
      /* o n ao incio da lista e encerrar        */
   if (!*lista) {
         /* Novo n  nico na lista */
      ptrNovo->anterior = ptrNovo;
         /* Ele aponta para si mesmo */
      ptrNovo->proximo = ptrNovo;
         /* Incio da lista apontar para novo n */
      *lista = ptrNovo;

      return; /* Servio completo */
   }

      /* Faz p apontar para o primeiro n da lista */
   p = *lista;

      /* Talvez a insero ocorra na */
      /* primeira posio da lista   */
   if (p->conteudo > conteudo) {
         /* (1) O n sucessor do novo n ser p */
      ptrNovo->proximo = p;

         /* (2) O n anterior ao novo n ser o */
         /* n correntemente anterior a p       */
      ptrNovo->anterior = p->anterior;

         /* (3) O n anterior a p ser */
         /* o antecessor do novo n    */
      p->anterior->proximo = ptrNovo;

         /* (4) O novo n  imediatamente */
         /* anterior ao n apontado por p */
      p->anterior = ptrNovo;

         /* (5) O novo n ser o primeiro da lista */
      *lista = ptrNovo;

      return; /* Servio completo */
   }

      /*                                                    */
      /* O local de insero ainda no foi encontrado e    */
      /* preciso procur-lo na lista, que tem pelo menos    */
      /* um n. Neste caso, o novo n ser inserido antes   */
      /* do primeiro n cujo contedo  maior do que aquele */
      /* do n a ser inserido. Se o contedo do n a ser    */
      /* inserido for maior do que o de qualquer n da      */
      /* lista, ele ser inserido ao seu final.             */
      /*                                                    */

      /* Faz p apontar para o segundo n (se houver). */
      /* Se no houver segundo n, p apontar para o  */
      /* primeiro n, pois a lista  circular.        */
   p = p->proximo;

      /* Procura o local de insero */
   while (p != *lista && p->conteudo <= conteudo)
      p = p->proximo;

      /* Neste instante, p aponta para o primeiro n adiante */
      /* do local da insero. I.e., o sucessor do n a ser  */
      /* inserido ser aquele para o qual p aponta.          */

      /* (1) O n sucessor do novo n ser p */
   ptrNovo->proximo = p;

      /* (2) O n anterior ao novo n ser o */
      /* n correntemente anterior a p       */
   ptrNovo->anterior = p->anterior;

      /* (3) O n anterior a p ser o antecessor do novo n */
   p->anterior->proximo = ptrNovo;

      /* (4) O novo n  imediatamente */
      /* anterior ao n apontado por p */
   p->anterior = ptrNovo;
}

/****
 *
 * RemoveListaDEC(): Remove de uma lista duplamente encadeada
 *                 circular um n que contm um determinado
 *                 valor especificado como parmetro
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro que
 *                             representa a lista na qual
 *                             ser feita a remoo
 *     conteudo (entrada) - contedo do n a ser removido
 *
 * Retorno: O contedo do n removido
 *
 ****/
tConteudo RemoveListaDEC(tListaDEC *lista, tConteudo conteudo)
{
   tConteudo itemRemovido; /* Guardar o contedo */
                           /* do n removido      */
   tListaDEC p = *lista; /* p apontar para o n */
                         /* a ser removido       */

   ASSEGURA( p, "Tentativa de remocao de elemento "
             "em lista vazia" );

      /* Verifica se o n a ser removido */
      /*  o primeiro da lista           */
   if ( p->conteudo == conteudo ) {
         /* O n anterior ao n que segue p ser    */
         /* aquele que correntemente  anterior a p */
      p->proximo->anterior = p->anterior;

         /* Desvia o n anterior do n a ser removido */
      p->anterior->proximo = p->proximo;

         /* O valor do ponteiro para o incio da lista  */
         /* depende do fato de a lista ter ficado vazia */
      *lista = (p->proximo == p) ? NULL : p->proximo;

         /* Guarda o valor do n removido da lista */
      itemRemovido = p->conteudo;

      free(p); /* Libera o espao ocupado pelo n removido */

      return itemRemovido; /* Servio completo */
   }

      /* O n a ser removido no  o primeiro */
   p = p->proximo; /* Passa para o prximo n (se houver) */

      /* Tenta encontrar o n no restante da lista */
   while ( p != *lista && p->conteudo != conteudo )
         /* p passa a apontar para o prximo n */
      p = p->proximo;

      /* Se p for igual a *lista, a lista foi */
      /* totalmente examinada sem que o n    */
      /* a ser removido tenha sido encontrado */
   ASSEGURA( p != *lista, "Tentativa de remocao de "
             "elemento inexistente" );

   /* Neste ponto, p aponta para o n a ser removido */

      /* O n anterior ao n que segue p ser    */
      /* aquele que correntemente  anterior a p */
   p->proximo->anterior = p->anterior;

      /* Desvia o n anterior do n a ser removido */
   p->anterior->proximo = p->proximo;

   /* Neste ponto, o n desejado no faz mais parte    */
   /* da lista, mas ele ainda ocupa espao em memria. */
   /* Portanto,  necessrio liberar este espao.      */

      /* Guarda contedo do n removido da lista antes */
      /* de liberar o espao ocupado pelo n           */
   itemRemovido = p->conteudo;

   free(p); /* Libera o espao ocupado pelo n */

   return itemRemovido;
}

/****
 *
 * EstaVaziaListaDEC(): Verifica se uma lista duplamente
 *                  encadeada circular est vazia
 *
 * Parmetros:
 *     lista (entrada) - a lista que ser verificada
 *
 * Retorno: 1, quando a lista est vazia, ou 0, em caso contrrio
 *
 ****/
int EstaVaziaListaDEC(tListaDEC lista)
{
   return !lista; /*  o mesmo que return lista == NULL */
}

/****
 *
 * BuscaListaDEC(): Retorna o endereo do contedo efetivo do
 *                  n que possui um determinado contedo numa
 *                  lista duplamente encadeada circular
 *
 * Parmetros:
 *      lista (entrada) - lista que ser pesquisada
 *      valor (entrada) - contedo do n procurado
 *
 * Retorno: Endereo dos dados que contm o referido contedo,
 *          se ele for encontrado. Caso contrrio, NULL.
 *
 ****/
tConteudo *BuscaListaDEC(tListaDEC lista, tConteudo valor)
{
   tListaDEC p = lista;  /* Faz p apontar para */
                         /* o incio da lista  */

      /* Verifica se o n procurado se */
      /* encontra no incio da lista   */
   if (p->conteudo == valor) {
      return &lista->conteudo; /* N encontrado */
   }

      /* O n ainda no foi encontrado. Verifica */
      /* se ele se encontra no restante da lista */
   do {
      p = p->proximo;
   } while (p != lista && p->conteudo != valor);

      /* Se 'p' for igual a 'lista', o n */
      /* procurado no foi encontrado     */
   if(p == lista) {
      return NULL; /* N no foi encontrado */
   }

      /* O n foi encontrado. Ento, retorna-se */
      /* o endereo de seu campo 'conteudo'.    */
   return &p->conteudo;
}

/****
 *
 * AlteraConteudoLDEC(): Altera o contedo efetivo de um n
 *                       de uma lista duplamente encadeada
 *                       circular
 *
 * Parmetros:
 *     lista (entrada) - lista que contm o n que ser alterado
 *     antigo (entrada) - o antigo valor
 *     novo (entrada) - o novo valor
 *
 * Retorno: 0, se a atualizao for bem sucedida, ou 1, se a
 *          atualizao no for possvel, porque o n no foi
 *          encontrado
 *
 * Observao: Se a lista for ordenada, uma chamada desta
 *             funo pode desfazer a ordenao da lista
 *
 ****/
int AlteraConteudoLDEC( tListaDEC lista, tConteudo antigo,
                        tConteudo novo )
{
   tConteudo *pConteudo; /* Ponteiro para o contedo */
                         /* que ser alterado        */

      /* Obtm o endereo do contedo efetivo do */
      /* n que casa com o parmetro 'antigo'    */
   pConteudo = BuscaListaDEC(lista, antigo);

      /* Se 'pConteudo' for NULL o n no foi encontrado */
   if (!pConteudo) {
      return 1; /* N no foi encontrado */
   }

         /*                                 */
         /* O n foi encontrado. Altera-se  */
         /* seu contedo e retorna-se zero. */
         /*                                 */

   *pConteudo = novo; /* Altera o contedo do n */

   return 0; /* Alterao completa */
}

/****
 *
 * DestroiListaDEC(): Libera o espao ocupado pelos ns de
 *                    uma lista duplamente encadeada circular,
 *                    tornando-a vazia
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a
 *                             lista duplamente encadeada
 *                             circular que ser destruda
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaDEC(tListaDEC *lista)
{
   tListaDEC p, /* Aponta para o prximo n a ser liberado */
             ultimo; /* Aponta para ltimo n a ser liberado */

   if (!*lista)
      return; /* Lista vazia no precisa ser destruda */

   p = *lista;
   ultimo = (*lista)->anterior;

   do {
         /* Passa para o prximo n  antes que o n     */
         /* corrente seja destrudo. No importa que    */
         /* o ponteiro para o incio da lista seja      */
         /* alterado, pois a lista ser mesmo destruda */
      *lista = (*lista)->proximo;

         /* Libera o espao ocupado pelo n corrente */
      free(p);

         /* *lista j est apontando para o prximo n */
      p = *lista;
   } while (p != ultimo);

   free(p); /* Faltou liberar o ltimo n */

   *lista = NULL; /* A lista ficou vazia */
}

/****
 *
 * ProximoListaDEC(): Encontra o prximo n de uma lista
 *                    duplamente encadeada circular
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *ProximoListaDEC(tListaDEC lista)
{
   static tNoListaDEC *proximoNo = NULL;

   if (lista) {
      if (!proximoNo) {
         proximoNo = lista;
      } else {
         proximoNo = proximoNo->proximo;

         if (proximoNo == lista) {
            proximoNo = NULL;
         }
      }
      return proximoNo ? &proximoNo->conteudo : NULL;
   } else {
      return NULL;
   }
}

/****
 *
 * AnteriorListaDEC(): Encontra o n anterior de uma lista
 *                     duplamente encadeada circular
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *AnteriorListaDEC(tListaDEC lista)
{
   static tNoListaDEC *noAnterior = NULL;

   if (lista) {
      if (!noAnterior) {
         noAnterior = lista->anterior;
      } else {
         noAnterior = noAnterior->anterior;

         if (noAnterior == lista->anterior) {
            noAnterior = NULL;
         }
      }
      return noAnterior ? &noAnterior->conteudo : NULL;
   } else {
      return NULL;
   }
}

