/****
 *
 * Ttulo: Lista Duplamente Encadeada Circular com Cabea
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Determina se um nmero inteiro positivo  primo
 *
 * Entrada: Valores inteiros positivos ou zero para encerrar o programa
 *
 * Sada: Uma mensagem indicando se o nmero  primo ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>    /* Entrada e sada   */
#include <stdlib.h>   /* Alocao dinmica */
#include "ASSEGURA.h" /* Macro ASSEGURA    */
#include "Lista.h"    /* Interface deste arquivo */

/***************** Definies de Funes ******************/

/****
 *
 * CriaListaDECCab(): Inicia uma lista duplamente encadeada
 *                    circular com cabea
 *
 * Parmetros: *lista (sada) - a lista que ser iniciada
 *
 * Retorno: Nada
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
tListaDECCab CriaListaDECCab(void)
{
   tListaDECCab L = malloc(sizeof(tNoLDECCab));

   ASSEGURA(L, "Impossivel alocar a cabeca da lista");

      /* Numa lista vazia, a cabea aponta para si prpria */
   L->anterior = L;
   L->proximo = L;

   return L;
}

/****
 *
 * ComprimentoListaDECCab(): Calcula o nmero de ns de uma
 *                           lista duplamente encadeada circular
 *                           com cabea
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a cabea da lista
 *                       cujo comprimento ser calculado
 *
 * Retorno: O nmero de ns da referida lista
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
int ComprimentoListaDECCab(tListaDECCab lista)
{
   int          tamanho = 0; /* Armazenar o tamanho da lista */
   tListaDECCab inicio = lista; /* Guarda a cabea da lista */

      /* Faz o ponteiro 'lista' apontar para o */
      /* primeiro n da lista (se ele existir) */
   lista = lista->proximo;

      /* Acessa cada n da lista e conta  */
      /* quantos ns so acessados.       */
    while (lista != inicio) {
      lista = lista->proximo; /* Passa para o prximo n */
      ++tamanho; /* Mais um n foi encontrado */
   }

   return tamanho;
}

/****
 *
 * InsereEmOrdemLDECCab(): Insere um novo n numa lista
 *                         duplamente encadeada circular
 *                         com cabea
 *
 * Parmetros:
 *    *lista (entrada/sada) - ponteiro para a lista na qual
 *                             ser feita a insero
 *     conteudo (entrada) - contedo do n que ser inserido
 *
 * Retorno: Nada
 *
 ****/
void InsereEmOrdemLDECCab( tListaDECCab *lista,
                           tConteudo conteudo )
{
   tNoLDECCab    *ptrNovo;
   tListaDECCab  p; /* p apontar para o n antes do */
                    /* qual ser feita a insero    */

      /* Tenta alocar um novo n */
   ptrNovo = malloc(sizeof(tNoLDECCab));

      /* Se o n no foi alocado, esta    */
      /* funo no tem mais nada a fazer */
   ASSEGURA( ptrNovo, "Nao foi possivel alocar um no'" );

      /* Armazena o contedo recebido */
      /* como parmetro no novo n    */
   ptrNovo->conteudo = conteudo;

      /* Armazena o contedo recebido como */
      /* parmetro na cabea da lista      */
   (*lista)->conteudo = conteudo;

      /* Faz p apontar para o primeiro n da lista */
   p = (*lista)->proximo;

      /* Procura o local de insero */
   while (p->conteudo < conteudo)
      p = p->proximo;

      /* Neste instante, p aponta para o primeiro n adiante */
      /* do local da insero. I.e., o sucessor do n a ser  */
      /* inserido ser aquele para o qual p aponta.          */

      /* O n sucessor do novo n ser p */
   ptrNovo->proximo = p;

      /* O n anterior ao novo n ser o */
      /* n correntemente anterior a p   */
   ptrNovo->anterior = p->anterior;

      /* O n anterior a p ser o antecessor do novo n */
   p->anterior->proximo = ptrNovo;

      /* (4) O novo n  imediatamente */
      /* anterior ao n apontado por p */
   p->anterior = ptrNovo;
}

/****
 *
 * RemoveListaDECCab(): Remove de uma lista duplamente encadeada
 *                    circular com cabea um n que contm um
 *                    determinado valor especificado como
 *                    parmetro
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro que
 *                             representa a lista na qual
 *                             ser feita a remoo
 *     conteudo (entrada) - contedo do n a ser removido
 *
 * Retorno: O contedo do n removido
 *
 ****/
tConteudo RemoveListaDECCab( tListaDECCab *lista, tConteudo conteudo )
{
   tConteudo itemRemovido; /* Guardar o contedo */
                           /* do n removido      */
   tListaDECCab p; /* p apontar para o n */
                   /* a ser removido       */

      /* Armazena o contedo recebido como */
      /* parmetro na cabea da lista      */
   (*lista)->conteudo = conteudo;

      /* Faz p apontar para o primeiro n da lista */
   p = (*lista)->proximo;

      /* Tenta encontrar o n na lista */
   while ( p->conteudo != conteudo )
      p = p->proximo;

      /* Se p for igual a *lista, a lista foi */
      /* totalmente examinada sem que o n    */
      /* a ser removido tenha sido encontrado */
   ASSEGURA( p != *lista, "Tentativa de remocao de "
             "elemento inexistente" );

   /* Neste ponto, p aponta para o n a ser removido */

      /* O n anterior ao n que segue p ser    */
      /* aquele que correntemente  anterior a p */
   p->proximo->anterior = p->anterior;

      /* Desvia o n anterior do n a ser removido */
   p->anterior->proximo = p->proximo;

   /* Neste ponto, o n desejado no faz mais parte    */
   /* da lista, mas ele ainda ocupa espao em memria. */
   /* Portanto,  necessrio liberar este espao.      */

      /* Guarda contedo do n removido da lista antes */
      /* de liberar o espao ocupado pelo n           */
   itemRemovido = p->conteudo;

   free(p); /* Libera o espao ocupado pelo n */

   return itemRemovido;
}

/****
 *
 * EstaVaziaListaDECCab(): Verifica se uma lista duplamente
 *                     encadeada circular com cabea est vazia
 *
 * Parmetros:
 *     lista (entrada) - a lista que ser verificada
 *
 * Retorno: 1, quando a lista est vazia, ou 0, em caso contrrio
 *
 ****/
int EstaVaziaListaDECCab(tListaDECCab lista)
{
   return lista->proximo == lista;
}

/****
 *
 * BuscaListaDECCab(): Retorna o endereo do contedo efetivo
 *                     do n que possui um determinado contedo
 *                     numa lista duplamente encadeada circular
 *                     com cabea
 *
 * Parmetros:
 *      lista (entrada) - lista que ser pesquisada
 *      valor (entrada) - contedo do n procurado
 *
 * Retorno: Endereo dos dados que contm o referido contedo,
 *          se ele for encontrado. Caso contrrio, NULL.
 *
 ****/
tConteudo *BuscaListaDECCab(tListaDECCab lista, tConteudo valor)
{
   tListaDECCab p = lista;  /* Faz p apontar para */
                         /* o incio da lista  */

      /* Armazena o contedo recebido como */
      /* parmetro na cabea da lista      */
   lista->conteudo = valor;

      /* Faz p apontar para o primeiro n da lista */
   p = lista->proximo;

      /* Tenta encontrar o n na lista */
   while ( p->conteudo != valor )
      p = p->proximo;

      /* Se 'p' for igual a 'lista', a lista */
      /* foi totalmente examinada sem que o  */
      /* n procurado tenha sido encontrado  */
   return p == lista ? NULL : &p->conteudo;
}

/****
 *
 * AlteraConteudoLDECCab(): Altera o contedo efetivo de um n
 *                          de uma lista duplamente encadeada
 *                          circular com cabea
 *
 * Parmetros:
 *     lista (entrada) - lista que contm o n que ser alterado
 *     antigo (entrada) - o antigo valor
 *     novo (entrada) - o novo valor
 *
 * Retorno: 0, se a atualizao for bem sucedida, ou 1, se a
 *          atualizao no for possvel, porque o n no foi
 *          encontrado
 *
 * Observao: Se a lista for ordenada, uma chamada desta
 *             funo pode desfazer a ordenao da lista
 *
 ****/
int AlteraConteudoLDECCab( tListaDECCab lista, tConteudo antigo,
                           tConteudo novo )
{
   tConteudo *pConteudo; /* Ponteiro para o contedo */
                         /* que ser alterado        */

      /* Obtm o endereo do contedo efetivo do */
      /* n que casa com o parmetro 'antigo'    */
   pConteudo = BuscaListaDECCab(lista, antigo);

      /* Se 'pConteudo' for NULL o n no foi encontrado */
   if (!pConteudo) {
      return 1; /* N no foi encontrado */
   }

         /*                                 */
         /* O n foi encontrado. Altera-se  */
         /* seu contedo e retorna-se zero. */
         /*                                 */

   *pConteudo = novo; /* Altera o contedo do n */

   return 0; /* Alterao completa */
}

/****
 *
 * DestroiListaDECCab(): Libera o espao ocupado pelos ns
 *                       de uma lista duplamente encadeada
 *                       circular com cabea, tornando-a
 *                       vazia e sem cabea
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a
 *                             lista duplamente encadeada
 *                             circular que ser destruda
 *
 * Retorno: Nada
 *
 ****/
void DestroiListaDECCab(tListaDECCab *lista)
{
   tListaDECCab p, /* Aponta para o prximo n a ser liberado */
                ultimo; /* Aponta para ltimo n a ser liberado */

      /* O ltimo n a ser liberado ser a cabea */
   ultimo = *lista;

      /* Faz p apontar para o primeiro n */
   p = (*lista)->proximo;

   while (p != ultimo) {
         /* Passa para o prximo n  antes   */
         /* que o n corrente seja destrudo */
      *lista = (*lista)->proximo;

         /* Libera o espao ocupado pelo n corrente */
      free(p);

         /* *lista j est apontando para o prximo n */
      p = *lista;
   }

   free(p); /* Faltou liberar o ltimo n */

   *lista = NULL; /* A lista ficou sem cabea e vazia */
}

/****
 *
 * ProximoListaDECCab(): Encontra o prximo n de uma lista
 *                       duplamente encadeada circular com cabea
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *ProximoListaDECCab(tListaDECCab lista)
{
   static tNoLDECCab *proximoNo = NULL;

   if (!EstaVaziaListaDECCab(lista)) {
      if (!proximoNo) {
         proximoNo = lista->proximo;
      } else {
         proximoNo = proximoNo->proximo;

         if (proximoNo == lista) {
            proximoNo = NULL;
         }
      }
      return proximoNo ? &proximoNo->conteudo : NULL;
   } else {
      return NULL;
   }
}

/****
 *
 * AnteriorListaDECCab(): Encontra o n anterior de uma lista
 *                        duplamente encadeada circular com cabea
 *
 * Parmetros:
 *     lista (entrada) - ponteiro para a lista
 *
 * Retorno: Endereo do contedo do prximo n
 *
 ****/
tConteudo *AnteriorListaDECCab(tListaDECCab lista)
{
   static tNoLDECCab *noAnterior = NULL;

   if (!EstaVaziaListaDECCab(lista)) {
      if (!noAnterior) {
         noAnterior = lista->anterior;
      } else {
         noAnterior = noAnterior->anterior;

         if (noAnterior == lista->anterior) {
            noAnterior = NULL;
         }
      }
      return noAnterior != lista ? &noAnterior->conteudo : NULL;
   } else {
      return NULL;
   }
}
