/****
 *
 * Ttulo: Invertendo uma Lista Simplesmente Encadeada
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Invertendo uma lista simplesmente encadeada de inteiros
 *
 * Entrada: No h
 *
 * Sada: Uma lista encadeada de inteiros antes e depois de ser invertida
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>  /* Entrada e sada   */
#include <stdlib.h> /* Alocao dinmica */

/******************* Definies de Tipos ******************/

   /* Tipo de n e tipo de ponteiro para n de uma lista simplesmente encadeada */
typedef struct rotNoLSE {
           int              dados;
           struct rotNoLSE *proximo;
        } tNoListaSE, *tListaSE;

/************************* Aluses ************************/

extern tListaSE CriaListaEncadedadaInts(int nNos);
extern int InsereNoLEInts(tListaSE *lista, int dados);
extern void ExibeListaEncadedadaInts(tListaSE lista);
extern tListaSE InverteListaSE(tListaSE *lista);

/****************** Definies de Funes *****************/

/****
 *
 * CriaListaEncadedadaInts(): Cria uma lista encadeada de inteiros
 *
 * Parmetros: nNos (entrada) - nmero de ns da listas
 *
 * Retorno: Ponteiro para lista lista criada, se no ocorrer erro;
 *          NULL, caso contrrio.
 *
 ****/
tListaSE CriaListaEncadedadaInts( int nNos )
{
   int i, erroInsercao; /* Indica se houve erro de insero na lista */
   tListaSE aLista = NULL;

      /* Cria o contedo de cada n e armazena-o na lista */
   for (i = nNos; i > 0; --i) {
         /* Tenta inserir n na lista */
      erroInsercao = InsereNoLEInts(&aLista, i);

         /* Se ocorreu erro de insero termina o lao */
      if (erroInsercao)
         break;
   }

      /* Verifica se ocorreu erro */
   if (erroInsercao)
      return NULL; /* Ocorreu algum erro */

   return aLista; /* Retorna o endereo inicial da lista */
}

/****
 *
 * ExibeListaEncadedadaInts(): Exibe uma lista encadeada de
 *                             inteiros do primeiro ao ltimo n
 *
 * Parmetros: lista (entrada) - a lista que ser exibida
 *
 * Retorno: Nada
 *
 ****/
void ExibeListaEncadedadaInts(tListaSE lista)
{
      /* Verifica se a lista est vazia */
   if(!lista) {
      printf("\nA lista esta' vazia\n");
      return; /* No h mais nada a fazer */
   }

   putchar('\n'); /* Embelezamento apenas */

      /* Visita cada n da lista exibindo a informao contida no n */
   while (lista) {
         /* Exibe cada campo dos dados do n */
      printf("%d ", lista->dados);

      lista = lista->proximo; /* Passa para o prximo n */

      if (lista) {
         printf("--> "); /* Embelezamento */
      }
   }

   putchar('\n'); /* Embelezamento apenas */
}

/****
 *
 * InsereNoLEInts(): Insere um novo n no incio de uma lista
 *                   encadeada de inteiros
 *
 * Parmetros:
 *     lista (entrada/sada) - endereo do ponteiro para a lista
 *                             na qual ser feita a insero
 *     dados (entrada) - ponteiro para os dados do n que ser inserido
 *
 * Retorno: Zero, se a insero for bem sucedida, ou um valor
 *          diferente de zero se a insero no for possvel.
 *
 ****/
int InsereNoLEInts(tListaSE *lista, int dados)
{
   tNoListaSE *ptrNovoNo; /* Apontar para o novo n alocado */

   ptrNovoNo = malloc(sizeof(tNoListaSE)); /* Tenta alocar um novo n */

   if(!ptrNovoNo) {
      return 1; /* Nao foi possivel alocar n */
   }

      /* Armazena no novo n os dados */
      /* recebidos como parmetro     */
   ptrNovoNo->dados = dados;

      /* O novo n apontar para o incio corrente da lista */
   ptrNovoNo->proximo = *lista;

      /* O incio da lista passa a apontar para o novo n */
   *lista = ptrNovoNo;

   return 0; /* Servio completo */
}

/****
 *
 * InverteListaSE(): Inverte uma lista encadeada
 *
 * Parmetros:
 *    *lista (entrada/sada) - ponteiro para a lista encadeada
 *                             que ser invertida
 *
 * Retorno: Endereo do primeiro n da lista invertida
 *
 * Observao: Esta funo independe do tipo de contedo
 *             efetivo armazenado em cada n da lista
 *
 ****/
tListaSE InverteListaSE(tListaSE *lista)
{
      /* Cada ponteiro a seguir aponta para o n... */
   tListaSE noAnterior = NULL,         /* ...anterior */
          noCorrente = *lista,       /* ...corrente */
          noSeguinte = NULL;         /* ...seguinte */

      /* Ver ilustraes dos passos nas figuras */
   while (noCorrente) {
      noSeguinte = noCorrente->proximo; /* Instruo 1 */
      noCorrente->proximo = noAnterior; /* Instruo 2 */
      noAnterior = noCorrente;          /* Instruo 3 */
      noCorrente = noSeguinte;          /* Instruo 4 */
   }

      /* Neste instante, o ponteiro 'noAnterior' aponta  */
      /* para o incio da lista invertida. Portanto,     */
      /* atribui-se seu valor ao ponteiro que representa */
      /* a lista e retorna-se o resultado.               */
   return *lista = noAnterior;
}

/****
 *
 * main(): Cria uma lista encadeada de elementos cujos contedos
 *         efetivos so do tipo int, e apresenta-a antes e depois
 *         de ser invertida
 *
 * Parmetros: Nenhum
 *
 * Retorno: 0, se no ocorrer nenhum erro; 1, em caso contrrio.
 *
 ****/
int main(void)
{
   tListaSE  lista = NULL; /* Apontar para a lista */

      /* Cria a lista encadeada */
   lista = CriaListaEncadedadaInts(10);

      /* Se a lista no pode ser criada */
      /* o programa no pode continuar  */
   if (!lista) {
      printf("\nOcorreu um erro durante a criacao da lista\n");
      return 1;
   }

   printf("\n\t\t*** Lista Original ***\n");
   ExibeListaEncadedadaInts(lista);

   InverteListaSE(&lista);

   printf("\n\t\t*** Lista Invertida ***\n");
   ExibeListaEncadedadaInts(lista);

   printf("\n\t\t*** Lista Original ***\n");
   ExibeListaEncadedadaInts(lista);

   InverteListaSE(&lista);

   printf("\n\t\t*** Lista Invertida ***\n");
   ExibeListaEncadedadaInts(lista);

   return 0; /* Tudo ocorreu bem */
}

/****************** Resultado do Programa *****************

                *** Lista Original ***

1 --> 2 --> 3 --> 4 --> 5 --> 6 --> 7 --> 8 --> 9 --> 10

                *** Lista Invertida ***

10 --> 9 --> 8 --> 7 --> 6 --> 5 --> 4 --> 3 --> 2 --> 1

**********************************************************/
