/****
 *
 * Ttulo: A Histria de Josephus
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2015
 * ltima modificao: 16/04/2015
 *
 * Descrio: Resolve o Problema de Josephus usando lista encadeada
 *
 * Entrada:
 *
 * Sada:
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/***************** Constantes Simblicas ******************/

#define MAX_NOME 25 /* Tamanho mximo de um nome de soldado de Josephus */

/****************** Definies de Tipos *******************/

typedef struct rotNoJosephus {
          char                  nome[MAX_NOME];
          struct rotNoJosephus *proximo;
        } tNoJosephus, *tListaJosephus;

/************************* Aluses ************************/

extern int LeCaractere(void);
extern int LeString(char *ar, int nElementos);
extern int LeOpcao(const char *opcoes);
extern void CriaListaDeSoldados(tListaJosephus *inicio);
extern char *Sobrevivente(tListaJosephus *inicio, int k);
extern void ExibeLista(tListaJosephus inicio);

/***************** Definies de Funes ******************/

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de caracteres lidos */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeCaractere(): L um caractere via teclado e remove os
 *                caracteres remanescentes no buffer
 *
 * Parmetros: Nenhum
 *
 * Retorno: O caractere lido
 *
 * Observao: Esta funo deixa o buffer limpo
 *
 ****/
int LeCaractere(void)
{
   int c, /* Armazenar cada caractere lido */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Volta para c se ocorrer erro de leitura */
inicio:
   c = getchar(); /* L o caractere digitado */

      /* Verifica se getchar() retornou EOF */
   if (c == EOF) {
      printf( "\a\n\t>>> Erro de leitura. "
              "Tente novamente\n\t> " );
      goto inicio; /* No faz mal algum */
   }

      /* Pelo menos '\n' se encontra no buffer */
   nResto = LimpaBuffer();

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n", nResto);
   }

   return c; /* Retorna o caractere lido */
}

/****
 *
 * LeString(): L um string via teclado deixando o buffer
 *             zerado
 *
 * Parmetros:
 *      ar[] (sada): array que armazenar o string lido
 *      nElementos (entrada): nmero mximo de elementos que
 *                            sero armazenados no array,
 *                            incluindo '\0'
 *
 * Retorno: O nmero de caracteres que o usurio digitou a
 *          mais, sem incluir '\n'
 *
 ****/
int LeString(char *ar, int nElementos)
{
   char *p; /* Usado para remover '\n' do string */
   int   nCarDeixados = 0; /* Conta o nmero de     */
                           /* caracteres excedentes */
                           /* no buffer             */

      /* L no mximo nElementos - 1 caracteres ou at   */
      /* encontrar '\n'. Quando fgets() retorna NULL,    */
      /* ocorreu erro ou tentativa de leitura alm do    */
      /* final do arquivo. Nesses casos, o lao continua */
      /* efetuando novas tentativas de leitura.          */
   while ( fgets(ar, nElementos, stdin) == NULL )  {
      printf( "\a\n\t>>> %s", "Erro de leitura. "
              "Tente novamente\n\t> " );
   }

      /* Faz p apontar para o caractere que      */
      /* antecede o caractere terminal do string */
   p = strchr(ar, '\0') - 1;

      /* Se o caractere '\n' foi lido, remove-o do   */
      /* string. Caso contrrio, remove-o do buffer. */
   if (*p == '\n') {
         /* Caractere '\n' faz parte do string */
      *p = '\0'; /* Sobrescreve '\n' com '\0' */
   } else { /* Usurio digitou caracteres demais */
      nCarDeixados = LimpaBuffer();
   }

      /* Retorna o nmero de caracteres que o usurio */
      /* digitou alm do esperado, sem incluir '\n'   */
   return nCarDeixados;
}

/****
 *
 * LeOpcao(): L, via teclado, um caractere que deve fazer
 *            parte de um string para ser considerado vlido
 *
 * Parmetros:
 *      opcoes (entrada) - string contendo os caracteres
 *                         vlidos
 *
 * Retorno: Um caractere vlido
 *
 * Observaes:
 *      1. Esta funo deixa o buffer virgem novamente
 *      2. Esta funo  denominada 'LeOpcao' porque o
 *         caractere lido e validado  restrito a um
 *         conjunto de caracteres (opes) representado
 *         pelo string recebido como parmetro
 *
 ****/
int LeOpcao(const char *opcoes)
{
   int op; /* Opo (caractere) escolhida pelo usurio */

      /* Enquanto o usurio no escolher uma opo */
      /* vlida, o lao a seguir no encerra       */
   while (1) {
         /* L o caractere digitado */
      op = LeCaractere();

         /* Verifica se o caractere  vlido */
      if (strchr(opcoes, op)) {
         break; /*  */
      } else { /* No  */
         printf( "\a\n\t>>> Opcao incorreta. "
                 "Tente novamente\n\t> " );
      }
   }

      /* Certamente, a opo retornada  vlida */
   return op;
}

/****
 *
 * LeNatural(): L um nmero natural via teclado e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres excedentes */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1 || num < 0) {
      printf( "\a\n\t>>> O valor digitado e' invalido. "
              "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Se for o caso, repreende o usurio por */
      /* ele ter digitado mais do que deveria   */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf("\t>>> %d caracteres foram descartados\n", nResto);
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * CriaListaDeSoldados(): L os nomes dos soldados de Josephus e cria uma
 *                        lista circular simplesmente encadeada contendo-os
 *
 * Parmetros:
 *      *inicio (entrada e sada) - ponteiro para o incio da lista
 *
 * Retorno: Nada
 *
 ****/
void CriaListaDeSoldados(tListaJosephus *inicio)
{
   tNoJosephus *pAux, /* Ponteiro usado na alocao de ns */
               *ultimo; /* Apontar sempre para o ltimo n */
   int          opcao;  /* Opo escolhida pelo usurio */

   do {
      pAux = malloc(sizeof(tNoJosephus)); /* Aloca espao para mais um n da lista */

         /* Solicita o nome do prximo soldado a ser includo na lista */
      printf( "\nDigite o nome do soldado (max = %d) >>> ", MAX_NOME - 1 );
      LeString(pAux->nome, MAX_NOME);

      pAux->proximo = NULL; /* Este ser o ltimo n da lista */

      if (*inicio == NULL)
         *inicio = pAux; /* Este  o primeiro n da lista */
      else
            /* O n que antes era o ltimo passa a apontar para o n mais recente */
         ultimo->proximo = pAux;

      ultimo = pAux; /* O n mais recentemente alocado agora  o ltimo */

         /* Verifica se ainda h soldado a ser includo */
      printf("\nDeseja acrescentar outro soldado (s/n)? ");
      opcao = LeOpcao("SsNn");
   } while (opcao == 'S' || opcao == 's');

      /* Torna a lista circular fazendo o ltimo n apontar para o primeiro n */
   ultimo->proximo = *inicio;
}

/****
 *
 * Sobrevivente(): Elimina soldados da lista de Josephus at
 *                 sobrar apenas um sobrevivente
 *
 * Parmetros:
 *      *inicio (entrada e sada) - ponteiro para o incio da lista
 *       salto (entrada) - o nmero de elementos que sero saltados a cada rodada
 *
 * Retorno: Ponteiro para o nome do elemento que se encontra
 *          na frente da lista ao final da chacina (i.e., o sobrevivente)
 *
 ****/
char *Sobrevivente(tListaJosephus *inicio, int salto)
{
   tNoJosephus *p, *q;
   int          i;

   q = p = *inicio;

      /* Remove ns da lista at que sobre apenas um n */
   while (p->proximo != p) {
         /* Elimina ns de salto em salto */
      for (i = 0; i < salto - 1; i++) {
         q = p;
         p = p->proximo;
      }

      q->proximo = p->proximo; /* Remove da lista o elemento para o qual p aponta */

         /* Informa qual  o nome do elemento removido */
      printf("%s foi morto\n", p->nome);

      free(p); /* Libera o espao ocupado pelo elemento removido */

      p = q->proximo;
   }

   *inicio = p; /* p aponta para o sobrevivente */

   return (p->nome); /* Retorna o endereo do nome do sobrevivente */
}

/****
 *
 * ExibeLista(): Exibe a lista de Josephus na tela
 *
 * Parmetros:
 *      inicio (entrada) - ponteiro para o incio da lista
 *
 * Retorno: Nada
 *
 ****/
void ExibeLista(tListaJosephus inicio)
{
   tNoJosephus *pAux;

      /********************************************/
      /*                                          */
      /* Visita sequencialmente cada n da lista  */
      /* exibindo na tela o campo nome de cada n */
      /*                                          */
      /********************************************/

   pAux = inicio;

   printf("\n%s\n", pAux->nome);

   pAux = pAux->proximo;

   while (inicio != pAux) {
      printf("%s\n", pAux->nome);
      pAux = pAux->proximo;
   }

   putchar('\n');
}

/****
 *
 * main(): Brinca de Josephus
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tListaJosephus inicio = NULL;
   int            salto;
   char          *sobrevive;

      /* Apresenta o programa ao usurio */
   printf( "\n\t>>> Este programa resolve o Problema de Josephus\n" );

      /* Cria a lista circular contendo os soldados */
   CriaListaDeSoldados(&inicio);

      /* Apresenta os soldados na lista antes da chacina */
   printf("\n\n>>> Soldados na lista <<<\n");
   ExibeLista(inicio);

      /* L o tamanho do salto */
   printf("Qual e' o numero de soldados a serem saltados? ");
   salto = LeNatural();

   printf("\n>>> Lista de infelizes <<<\n\n");

      /* Apresenta os soldados  medida em que so mortos */
      /* e recebe o nome do soldado que sobrevive         */
   sobrevive = Sobrevivente(&inicio, salto);

   printf( "\n>>> O soldado sobrevivente e': %s\n", sobrevive );

      /* Neste ponto, a lista contm apenas um n que  */
      /* no  mais necessrio e, portanto,  liberado */
   free(inicio);

   return 0;
}

/******************* Exemplo de Execuo ******************

Digite o nome do soldado (max = 24) >>> Fernando

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Luiz Josephus

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Edinho

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Gleisi

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Jader

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Delcidio

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Renan

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Romero

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Roseana

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Tiao

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Valdir

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Dirceu

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Lindbergh

Deseja acrescentar outro soldado (s/n)? s

Digite o nome do soldado (max = 24) >>> Candido

Deseja acrescentar outro soldado (s/n)? n

>>> Soldados na lista <<<

Fernando
Luiz Josephus
Edinho
Gleisi
Jader
Delcidio
Renan
Romero
Roseana
Tiao
Valdir
Dirceu
Lindbergh
Candido

Qual e' o numero de soldados a serem saltados? 3

>>> Lista de infelizes <<<

Edinho foi morto
Delcidio foi morto
Roseana foi morto
Dirceu foi morto
Fernando foi morto
Jader foi morto
Tiao foi morto
Candido foi morto
Renan foi morto
Lindbergh foi morto
Romero foi morto
Gleisi foi morto
Valdir foi morto

>>> O soldado sobrevivente e': Luiz Josephus

**********************************************************/
