/****
 *
 * Ttulo: Converso e Avaliao de Expresses
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 10/04/2013
 * ltima modificao: 10/04/2013
 *
 * Descrio: Converte expresses infixas em sufixas e
 *            as avalia
 *
 * Entrada: Expresses infixas contendo nmeros inteiros,
 *          parnteses e os operadores: _ (menos unrio),
 *          - (subtrao), + (soma), * (multiplicao) e
 *          / (diviso)
 *
 * Sada: Expresses sufixas correspondentes ou resultados
 *        de avaliaes de expresses introduzidas
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>      /* Entrada e Sada                */
#include <stdlib.h>     /* Alocao dinmica              */
#include "Expressoes.h" /* Interface do mdulo Expressoes */
#include "leitura.h"    /* Funes de leitura via teclado */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo de uma expresso */
   /* introduzida via teclado         */
#define TAM_MAX 80

/********************* Funo main() **********************/

/****
 *
 * main(): Converte expresses infixas em sufixas e
 *         avalia-as
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main (void)
{
   char  infixa[TAM_MAX + 1],/* Armazena expresses       */
                             /* introduzidas pelo usurio */
        *sufixa; /* Apontar para expresses sufixas */
   int  opcao; /* Opo do usurio */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa avalia expressoes"
           "\n\t>>> aritmeticas inteiras e converte-as"
           "\n\t>>> em forma sufixa\n" );

      /* O lao termina quando o usurio */
      /* decidir encerrar o programa     */
   while (1) {
         /* L a expresso */
      printf( "\n\t>>> Digite a expressao "
              "(max = %d caracteres):\n\t    > ", TAM_MAX );
      LeString(infixa, TAM_MAX + 1);

         /* Apresenta o menu de opes do programa */
      printf("\n\t>>> Escolha uma opcao:"
             "\n\t\t1. Converte em expressao sufixa"
             "\n\t\t2. Avalia a expressao"
             "\n\t\t3. Introduz outra expressao"
             "\n\t\t4. Encerra o programa");

         /* O lao termina quando o usurio */
         /* escolher a opo '3' ou '4'     */
      do {
            /* L a opo do usurio */
         printf("\n\n\t>>> Sua opcao: ");
         opcao = LeOpcao("1234");

            /* A opo '4' encerra o programa */
         if (opcao == '4') {
               /* Despede-se do usurio */
            printf( "\n\t>>> Obrigado por usar este"
                    " programa.\n" );

            return 0; /* Encerra o programa */
         }

            /* Lida com as opes '1' e '2' */
         if (opcao == '1') {
               /* Obtm a expresso sufixa correspondente */
            sufixa = ExpressaoSufixa(infixa);

               /* Apresenta a expresso na forma sufixa */
            printf( "\n\t>>> Expressao Sufixa: %s\n",
                    sufixa );

               /* O espao ocupado pela expresso  */
               /* sufixa foi alocado dinamicamente */
               /* e agora deve ser liberado        */
            free(sufixa);
         } else if (opcao == '2') {
               /* Calcula e apresenta o  */
               /* resultado da expresso */
            printf( "\n\t>>> Resultado da expressao: %d\n",
                    AvalExpressaoInfixa(infixa) );
         }
      } while (opcao != '3');
   }

   /****************************************************/
   /*                                                  */
   /* O fluxo de execuo no deve nunca chegar at    */
   /* aqui, j que o programa encerra no corpo do lao */
   /*                                                  */
   /****************************************************/

   printf("\nEsta mensagem nunca devera' aparecer!\n");

   return 0;
}

/******************* Exemplos de Execuo ******************

Exemplo 1:
=========

        >>> Este programa avalia expressoes
        >>> aritmeticas inteiras e converte-as
        >>> em forma sufixa

        >>> Digite a expressao (max = 80 caracteres):
            > 2033040405405*4555

        >>> Escolha uma opcao:
                1. Converte em expressao sufixa
                2. Avalia a expressao
                3. Introduz outra expressao
                4. Encerra o programa

        >>> Sua opcao: 1

        >>> Expressao Sufixa: 2033040405405 4555 *


        >>> Sua opcao: 2


        >>> Overflow!!! <<<

Exemplo 2:
=========

        >>> Este programa avalia expressoes
        >>> aritmeticas inteiras e converte-as
        >>> em forma sufixa

        >>> Digite a expressao (max = 80 caracteres):
            > 20330490*45559990

        >>> Escolha uma opcao:
                1. Converte em expressao sufixa
                2. Avalia a expressao
                3. Introduz outra expressao
                4. Encerra o programa

        >>> Sua opcao: 1

        >>> Expressao Sufixa: 20330490 45559990 *


        >>> Sua opcao: 2

        >>> Resultado da expressao: -20927556


        >>> Sua opcao: 4

        >>> Obrigado por usar este programa.

   [Note que ocorreu overflow no detectado no resultado.]

Exemplo 3:
=========

        >>> Este programa avalia expressoes
        >>> aritmeticas inteiras e converte-as
        >>> em forma sufixa

        >>> Digite a expressao (max = 80 caracteres):
            > (2 + 3)*(_5 + 1)/2

        >>> Escolha uma opcao:
                1. Converte em expressao sufixa
                2. Avalia a expressao
                3. Introduz outra expressao
                4. Encerra o programa

        >>> Sua opcao: 1

        >>> Expressao Sufixa: 2 3 + 5 _ 1 + * 2 /


        >>> Sua opcao: 2

        >>> Resultado da expressao: -10


        >>> Sua opcao: 3

        >>> Digite a expressao (max = 80 caracteres):
            > 2*x + 1

        >>> Escolha uma opcao:
                1. Converte em expressao sufixa
                2. Avalia a expressao
                3. Introduz outra expressao
                4. Encerra o programa

        >>> Sua opcao: 1


Erro em ProximoToken(): caractere espurio encontrado

[Programa foi abortado devido  presena de x na expresso]

***********************************************************/
