/****
 *
 * Ttulo: rvores de Deciso
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/04/2014
 * ltima modificao: 16/04/2014
 *
 * Descrio: Implementa uma soluo para o problema das
 *            oito moedas usando uma rvore de deciso
 *
 * Entrada: Pesos das moedas
 *
 * Sada: v. Exemplo de Execuo
 *
 ****/

#include <stdio.h> /* Entrada e sada */

#define NUM_MOEDAS 8 /* Nmero de moedas */

typedef enum {A, B, C, D, E, F, G, H} tMoeda;

/****
 *
 * LimpaBuffer(): Limpa o buffer associado ao meio de
 *                entrada padro (stdin)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero de caracteres descartados, com
 *          exceo do caractere '\n'.
 *
 ****/
static int LimpaBuffer(void)
{
   int carLido, /* Armazena cada caractere lido */
       nCarLidos = 0; /* Conta o nmero de */
                      /* caracteres lidos  */

      /* L e descarta cada caractere lido at    */
      /* encontrar '\n' ou getchar() retornar EOF */
   do {
      carLido = getchar(); /* L um caractere */
      ++nCarLidos; /* Mais um caractere foi lido */
   } while ((carLido != '\n') && (carLido != EOF));

      /* O ltimo caractere lido foi '\n' ou  */
      /* EOF e no deve ser considerado sobra */
   return nCarLidos - 1;
}

/****
 *
 * LeNaturalPositivo(): L um nmero natural positivo via
 *                      teclado e deixa o buffer intacto
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNaturalPositivo(void)
{
   int num, /* O nmero lido */
       teste, /* Valor retornado por scanf() */
       nResto = 0; /* Nmero de caracteres */
                   /* excedentes           */

   /* Desvia para c se o valor for invlido */
inicio:
      /* Tenta ler um valor vlido */
   teste = scanf("%d", &num);

      /* Se no ocorreu erro de leitura ou de final */
      /* de arquivo, h caracteres remanescentes    */
      /* que precisam ser removidos                 */
   if (teste != EOF) {
      nResto = LimpaBuffer();
   }

      /* Enquanto o valor retornado por scanf()  */
      /* no indicar que um valor vlido foi     */
      /* lido continua tentando obter esse valor */
   while(teste != 1 || num < 1) {
      printf("\a\n\t>>> O valor digitado e' invalido. "
                      "Tente novamente\n\t> " );
      goto inicio; /* No causa dano algum */
   }

      /* Repreende o usurio se ele digitou demais */
   if (nResto == 1) {
      printf("\t>>> Um caractere foi descartado\n");
   } else if (nResto > 1) {
      printf( "\t>>> %d caracteres foram descartados\n",
              nResto );
   }

      /* O valor retornado certamente  vlido */
   return num;
}

/****
 *
 * Compara(): Compara moedas e exibe o resultado da
 *            comparao
 *
 * Parmetros:
 *      x (entrada) - moeda a ser comparada
 *      y (entrada) - uma moeda
 *      z (entrada) - moeda padro
 *      p[] (entrada) - array contendo os pesos das moedas
 *
 * Retorno: Nada
 *
 ****/
void Compara(tMoeda x, tMoeda y, tMoeda z, const int p[])
{
   if (p[x] > p[z])
      printf("\n>>> A moeda %c e' a mais pesada", 'A' + x);
   else if (p[x] < p[z])
      printf("\n>>> A moeda %c e' a mais leve", 'A' + y);
   else
      printf("\n>>> As moedas tem o mesmo peso");
}

/****
 *
 * main(): Resolve o problema das oito moedas
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int    pesos[NUM_MOEDAS];
   tMoeda M;

   printf("Problema das Oito Moedas\n");
   printf("======== === ==== ======\n");

   for (M = A; M <= H; ++M) {
      printf("\nPeso da moeda %c: ", 'A' + M);
      pesos[M] = LeNaturalPositivo();
   }

   if ( pesos[A] + pesos[B] + pesos[C] ==
        pesos[D] + pesos[E] + pesos[F] )
      if (pesos[G] > pesos[H])
         Compara(G, H, A, pesos);
      else
         Compara(H, G, A, pesos);
   else if ( pesos[A] + pesos[B] + pesos[C] >
             pesos[D] + pesos[E] + pesos[F] )
      if (pesos[A] + pesos[D] == pesos[B] + pesos[E])
         Compara(C, F, A, pesos);
      else if (pesos[A] + pesos[D] > pesos[B] + pesos[E])
         Compara(A, E, B, pesos);
      else
         Compara(B, D, A, pesos);
   else if ( pesos[A] + pesos[B] + pesos[C] <
             pesos[D] + pesos[E] + pesos[F] )
      if (pesos[A] + pesos[D] == pesos[B] + pesos[E])
         Compara(F, C, A, pesos);
      else if (pesos[A] + pesos[D] > pesos[B] + pesos[E])
         Compara(D, B, A, pesos);
      else
         Compara(E, A, B, pesos);

   return 0;
}

/******************* Exemplo de Execuo ******************

Problema das Oito Moedas
======== === ==== ======

Peso da moeda A: 2

Peso da moeda B: 2

Peso da moeda C: 2

Peso da moeda D: 2

Peso da moeda E: 2

Peso da moeda F: 2

Peso da moeda G: 6

Peso da moeda H: 2

>>> A moeda G e' a mais pesada

**********************************************************/
