/****
 *
 * Exerccio de Programao EP11.18
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/08/2012
 * ltima modificao: 26/08/2012
 *
 * Descrio: Conta o nmero de ocorrncias de letras num arquivo de texto
 *
 * Entrada: Contedo do arquivo "ListaDePalavras.txt"
 *
 * Sada: Nmero de ocorrncia de cada letra do arquivo
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada             */
#include <ctype.h>   /* Classificao de caracteres */

/***************** Constantes Simblicas ******************/

   /* Tamanho mximo de uma palavra */
#define TAM_MAX_PALAVRA  30

   /* Numero de letras no alfabeto romano */
#define NUMERO_DE_LETRAS  26

   /* Nome do arquivo que ser utilizado */
#define NOME_ARQUIVO "ListaDePalavras.txt"

/************************* Aluses ************************/

extern void MaiorMenorOcorrencia(const int contaLetras[]);
extern int TotalDeLetras(const int contaLetras[]);
extern void ApresentaFrequenciaDeLetras(const int ar[]);
extern int IndiceDeLetra(int c);

/***************** Definies de Funes ******************/

/****
 *
 * MaiorMenorOcorrencia(): Apresenta as letras que possuem a maior e menor
 *                         ocorrncias no array de ocorrncias
 *
 * Parmetros:
 *      contaLetras (entrada) - array contendo o nmero
 *                              de ocorrncias de cada letra num arquivo
 *
 * Retorno: Nada.
 *
 *
 ****/
void MaiorMenorOcorrencia(const int contaLetras[])
{
   int maior,       /* Maior frequncia */
       maiorIndice, /* ndice da maior frequncia */
       menor,       /* Menor frequncia */
       menorIndice, /* ndice da menor frequncia */
       i;

      /* Inicia a menor e a maior frequncias */
      /* com a primeira frequncia do array   */
   maior = menor = contaLetras[0];

      /* Inicia os ndices da menor e da maior     */
      /* frequncia com o primeiro ndice do array */
   maiorIndice = menorIndice = 0;

   /************************************************/
   /*                                              */
   /* Examina cada frequncia do array. Se for     */
   /* encontrada uma frequncia maior do que       */
   /* aquela correntemente considerada maior, a    */
   /* frequncia encontrada passa a ser a maior e  */
   /* o ndice da maior frequncia passa a ser o   */
   /* ndice desta frequncia. Idntico raciocnio */
   /*  aplicada nas determinaes da menor        */
   /* frequncia e seu ndice.                     */
   /*                                              */
   /************************************************/

   for (i = 1; i < NUMERO_DE_LETRAS; ++i) {
      if (contaLetras[i] > maior) {
         maior = contaLetras[i];
         maiorIndice = i;
      }

      if (contaLetras[i] < menor) {
         menor = contaLetras[i];
         menorIndice = i;
      }
   }

   /**************************************************/
   /*                                                */
   /* A letra que corresponde a um dado ndice      */
   /* obtida somando-se o ndice com 'A', pois esta  */
   /* letra corresponde ao ndice 0, 'B' corresponde */
   /* ao ndice 1, e assim por diante.               */
   /*                                                */
   /**************************************************/

   printf( "\n>>> Letra com maior frequencia (%d vezes):"
           " '%c'", maior, maiorIndice + 'A' );

   printf( "\n>>> Letra com menor frequencia (%d vezes):"
           " '%c'\n", menor, menorIndice + 'A' );
}

/****
 *
 * TotalDeLetras(): Calcula o nmero total de ocorrncias de
 *                  letras no array recebido como parmetro
 *
 * Parmetros:
 *      contaLetras (entrada) - array contendo o nmero de ocorrncias de cada
 *                              letra num arquivo
 *
 * Retorno: O nmero total de ocorrncias de letras
 *          no array recebido como parmetro
 *
 ****/
int TotalDeLetras(const int contaLetras[])
{
   int i, total = 0;

   for (i = 0; i < NUMERO_DE_LETRAS; ++i) {
      total = total + contaLetras[i];
   }

   return total;
}

void CriaCase(int CAR, int car, double freq)
{
   printf("\n      case '%c':"
          "\n      case '%c':"
          "\n         return %5.2f;\n", CAR, car, freq );
}

/****
 *
 * ApresentaFrequenciaDeLetras(): Apresenta o nmero de ocorrncias de cada
 *                                letra no array
 *
 * Parmetros:
 *      contaLetras (entrada) - array contendo o nmero de ocorrncias de cada
 *                              letra num arquivo
 *
 * Retorno: Nada.
 *
 ****/
void ApresentaFrequenciaDeLetras(const int contaLetras[])
{
   int ocorrencias, c, total;

   total = TotalDeLetras(contaLetras);

      /* Determina o nmero de ocorrncias de */
      /* cada letra no array contaLetras[]    */
   for (c = 'A'; c <= 'Z'; ++c) {

         /* Verifica se o caractere  uma letra */
      if (isalpha(c)) { /* Caractere  uma letra */
            /* Obtm o nmero de ocorrncias da letra */
         ocorrencias = contaLetras[IndiceDeLetra(c)];

         CriaCase(c, tolower(c), ((double) ocorrencias/total)*100);
      }
   }
}

/****
 *
 * IndiceDeLetra(): Determina o ndice correspondente
 *                  a uma letra no array que conta
 *                  frequncias de letras
 *
 * Parmetros:
 *      c (entrada) - caractere que supostamente representa uma letra
 *
 * Retorno: O referido ndice, se o parmetro representar
 *          uma letra. Caso contrrio, -1.
 *
 ****/
int IndiceDeLetra(int c)
{
   /*****************************************************/
   /*                                                   */
   /* Se o caractere for letra, converte-a em maiscula */
   /* e subtrai seu valor de 'A'. Assim, 'A' (ou 'a')   */
   /* tem ndice 0, 'B' (ou 'b') tem ndice 1, etc.     */
   /*                                                   */
   /*****************************************************/

      /* Verifica se o caractere  letra */
   if (isalpha(c)) { /*  letra */
      return toupper(c) - 'A';
   } else {  /* No  letra */
      return -1;
   }
}

/****
 *
 * main(): Conta o nmero de ocorrncias de letras
 *         num arquivo de texto e informa quais so
 *         as letras que possuem a maior e a menor
 *         frequncia.
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   int   c, indice, contaLetras[NUMERO_DE_LETRAS] = {0};
   FILE *stream;

   stream = fopen(NOME_ARQUIVO, "r");

   if (!stream) {
      printf( "\n\t>>> Impossivel abrir arquivo \"%s\"\n",
              NOME_ARQUIVO );
      return 1;
   }

      /* O lao encerra quando o final do arquivo */
      /* for atingido ou ocorrer erro de leitura  */
   while (1) {
         /* L um caractere */
      c = fgetc(stream);

         /* Verifica se ocorreu erro */
      if (ferror(stream)) { /* Ocorreu erro de leitura */
         printf( "\n\t>>> Erro de leitura no arquivo "
                 "\"%s\"\n", NOME_ARQUIVO );

         fclose(stream); /* Fecha arquivo antes de partir */

         return 1; /* Encerra o programa */
      }

         /* Verifica se final do arquivo foi atingido */
      if (feof(stream)) { /* Acabou o arquivo */
         break; /* Encerra o lao */
      }

         /* Se o caractere lido for uma letra, obtm seu */
         /* ndice no array que armazena as ocorrncias  */
      if (isalpha(c)) {
            /* Obtm o ndice no array de ocorrncias */
            /* que corresponde  letra corrente       */
         indice = IndiceDeLetra(c);

            /* O ndice retornado no pode jamais */
            /* ser negativo neste caso, pois o    */
            /* caractere  uma letra              */
         if (indice < 0) {
            printf("\a\nEste programa contem um erro!\n");
            return 1;
         }

            /* Incrementa o nmero de ocorrncias */
            /* da letra corrente                  */
         ++contaLetras[indice];
      }
   }

   fclose(stream); /* Fecha arquivo */

      /* Apresenta o resultado */
   ApresentaFrequenciaDeLetras(contaLetras);

   return 0;
}

/******************* Exemplo de Execuo ******************

      case 'A':
      case 'a':
         return 14.26;

      case 'B':
      case 'b':
         return  1.54;

      case 'C':
      case 'c':
         return  5.54;

      case 'D':
      case 'd':
         return  4.13;

      case 'E':
      case 'e':
         return  9.80;

      case 'F':
      case 'f':
         return  1.34;

      case 'G':
      case 'g':
         return  1.76;

      case 'H':
      case 'h':
         return  0.92;

      case 'I':
      case 'i':
         return  9.20;

      case 'J':
      case 'j':
         return  0.31;

      case 'K':
      case 'k':
         return  0.02;

      case 'L':
      case 'l':
         return  4.03;

      case 'M':
      case 'm':
         return  3.25;

      case 'N':
      case 'n':
         return  5.57;

      case 'O':
      case 'o':
         return 10.24;

      case 'P':
      case 'p':
         return  2.61;

      case 'Q':
      case 'q':
         return  0.42;

      case 'R':
      case 'r':
         return  9.08;

      case 'S':
      case 's':
         return  4.94;

      case 'T':
      case 't':
         return  5.62;

      case 'U':
      case 'u':
         return  3.02;

      case 'V':
      case 'v':
         return  1.47;

      case 'W':
      case 'w':
         return  0.01;

      case 'X':
      case 'x':
         return  0.38;

      case 'Y':
      case 'y':
         return  0.02;

      case 'Z':
      case 'z':
         return  0.53;

**********************************************************/
